#ifndef AITEXTGENERATEDPIC_H
#define AITEXTGENERATEDPIC_H

#include <QObject>
#include <QWidget>
#include <QThread>
#include <QStringList>
#include <QList>
#include <kylin-ai/ai-base/modelconfig.h>
#include <kylin-ai/ai-base/vision.h>
#include <kylin-ai/config.h>

/**
 * @brief The AIModeDataThread class
 * 线程类，用于跑AI的服务
 */
class AIModeDataThread : public QObject
{
    Q_OBJECT

public:
    AIModeDataThread();
    ~AIModeDataThread();

    bool setSessionStatus();
    bool someFunctionInThread();

    void setAIData (QString str, QSize size, int quantity, VisionImageStyle Style){
        textBox = str;
        AIImageSize = size;
        quantityNum = quantity;
        ImageStyle = Style;
    }

signals:
    void finishedSignal(VisionImageData *imageData, int errorCode);

private:
    void sendErrorSignal(int errorCode){
        emit finishedSignal(nullptr, errorCode);
    }
    static void generateImageCallback(VisionImageData imageData, void* userData);

private:
    QPixmap pixmap;
    int     quantityNum = 4; //数量
    QSize       AIImageSize;
    QString     textBox;
    VisionImageStyle ImageStyle;
    bool          m_sessionFlag = false;
    VisionSession session;
};

class AITextGeneratedPic : public QWidget
{
    Q_OBJECT

public:
    AITextGeneratedPic();
    ~AITextGeneratedPic();

    bool getWhetherToDrawForAI(QString message);
    void wordInformationProcessing(QString message);
    QString AIGeneratesErrorHandling(int errorCode);

    void startThread() {
        if (m_AiMode->setSessionStatus()){
                    m_AiMode->setAIData(textBox, AIImageSize, quantityNum, ImageStyle);
                    m_AiMode->someFunctionInThread();
        }
    }
    void onThreadFinished(VisionImageData *imageData, int errorCode);

signals:
    void pictureUpdateSignal(VisionImageData *imageData, QString errorMessage);

private:
    int     quantityNum = 1; //数量
    int     multipleNum = 1; //像素倍数
    QSize   AIImageSize;    //图片大小
    QString textBox;        //图像描述文字

    QStringList wordJudgment = {"绘制","创造","生成","打造","制作","创作","显现","呈现",
                                "产生","输出","创建","绘制","生成","设计","描绘","展示",
                                "制作","需要","想要","画","构思","表现","呈现","开发",
                                "可视化","模拟","想看","形成","渲染","合成","图示","绘画"};

    QStringList mainFont = {"图片","图像","插图","照片","画作","海报","草图"};


    QStringList quantityString = {"一张","两张","四张","八张","1张","2张","4张","8张","一个","两个","四个","八个","1个","2个","4个","8个","一幅","1幅"};

    QStringList shapeString = {"长方形","正方形","16:9","1:1","9:16","16：9","1：1","9：16"};

    QStringList pixelString = {"默认","普通","高清","默认像素","普通像素","高清像素"};

//    QStringList styleString = {"探索无限","古风","二次元","写实风格","浮世绘",
//                               "低多边形","未来主义","像素风格","概念艺术",
//                               "赛博朋克","洛丽塔风格","巴洛克风格","超现实主义",
//                               "水彩画","蒸汽波艺术","油画","卡通画"};

    VisionImageStyle  ImageStyle = EXPLORE_INFINITY; //风格样式
    AIModeDataThread  *m_AiMode;  //线程类
};

#endif // AITEXTGENERATEDPIC_H
