#include <QDir>
#include <QFileInfo>
#include <QMessageBox>
#include <QStringLiteral>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QProcess>
#include <QDesktopServices>
#include <QFileDialog>
#include "channelmessage.h"

struct MeetingItem{
    QString name;
    QString path;
    QString time;
//    QString duration;
};

ChannelMessage::ChannelMessage(QObject *parent) : QObject(parent)
{

}

ChannelMessage::~ChannelMessage()
{}

void ChannelMessage:: sendMessageFromJs(const QString &jsonMsgHead, const QString &message) {
    qDebug() << "jsonMsgHead:" << jsonMsgHead << "  Called from js with message:" << message;
    emit sigMessageFromJs(jsonMsgHead, message);
}

void ChannelMessage:: clearMessageFromJs() {
    emit sigClearFromJs();
}

void ChannelMessage:: updateReply(const QString &jsonMsgHead, const QString &message) {
    emit sigUpdateReply(jsonMsgHead, message);
}

void ChannelMessage::stopReply()
{
    emit sigStopReply();
}

void ChannelMessage::saveMessageList()
{
    emit sigSaveMessageList();
}

void ChannelMessage::getMessageList()
{
    emit sigGetMessageList();
}

void ChannelMessage::startVoiceInput()
{
    emit sigStartRecord();
}

void ChannelMessage::stopVoiceInput()
{
    emit sigStopRecord();
}

void ChannelMessage::cancelVoiceInput()
{
    emit sigCancelRecord();
}

void ChannelMessage::startRuntime(QString lan, QString tartget)
{
    emit sigStartRuntime(lan, tartget);
}

void ChannelMessage::stopRuntime()
{
    emit sigStopRuntime();
}


void ChannelMessage::startRead(const QString &msg, QString index)
{

    emit sigStartRead(msg, index.toInt());
}


void ChannelMessage::pauseRead(QString index)
{

    emit sigPauseRead(index.toInt());
}


void ChannelMessage::continueRead(QString index)
{
    emit sigContinueRead(index.toInt());
}

void ChannelMessage::clearDictation()
{
    emit sigClearDictation();
}

void ChannelMessage::copyDictation()
{
    emit sigCopyDictation();
}

void ChannelMessage::startMeeting()
{
    emit sigStartMeeting();
}

void ChannelMessage::pauseMeeting()
{
    emit sigPauseMeeting();
}

void ChannelMessage::continueMeeting()
{
    emit sigContinueMeeting();
}

void ChannelMessage::finishMeeting()
{
    emit sigFinishMeeting();
}


void ChannelMessage::editName(QString originalName)
{
    emit sigRename(originalName);
}

void ChannelMessage::getMeetingList()
{
    //扫描本地文件生成List
    QString documentsDir = QDir::homePath() + "/文档";
    QString directoryPath = documentsDir + "/kylin-aiassistant/meeting";
    QDir dir(directoryPath);
    if (!dir.exists()) {
        qDebug() << "目录不存在：" << directoryPath;
        return;
    }

    QJsonArray jsonArray;
    QStringList filters;
    filters << "*.txt" << "*.temp" ; // 设置过滤器以只获取txt文件和temp临时文件
    QStringList entries = dir.entryList(filters, QDir::Files); // 获取文件列表
    foreach (const QString &entry, entries) {       
        QFileInfo fileInfo(dir.filePath(entry));
        QString fileName = fileInfo.fileName();
        int indexUnder = fileName.indexOf('_');
        int indexDot = fileName.indexOf('.');

        QJsonObject jsonObject;
        QString index = QString::number(entries.indexOf(entry));
        jsonObject["index"] = index;
        jsonObject["path"] = directoryPath + "/" + fileName;
        jsonObject["name"] = fileName.mid(0,indexDot);
        QString date = fileName.mid(indexUnder + 1,  4) + "/";
        date += fileName.mid(indexUnder + 5, 2) + "/";
        date += fileName.mid(indexUnder + 7, 2) + "/ · ";
        date += fileName.mid(indexUnder + 10, 2) + ":";
        date += fileName.mid(indexUnder + 12, 2) +":";
        date += fileName.mid(indexUnder + 14,2);
        jsonObject["time"] = date;
        if(fileInfo.suffix() == "temp")
        {
            jsonObject["generating"] = "true";
         }
        else
        {
            jsonObject["generating"] = "false";
         }
        jsonArray.append(jsonObject);

    }

    QString result = QJsonDocument(jsonArray).toJson(QJsonDocument::Indented);

    emit sigMeetingList(result);
}

void ChannelMessage::openMeetingFile(QString path)
{
    QProcess process;
    QString command = "pluma";
    QStringList arguments;
    arguments << path;
    process.startDetached(command, arguments);
    process.waitForFinished(-1);
    // 检查进程是否正常退出
    if (process.exitStatus() == QProcess::NormalExit) {
        qDebug() << "Pluma opened the file successfully!";
    } else {
        qDebug() << "Failed to open the file with Pluma!";
    }

}

void ChannelMessage::openMettingFolder(QString path)
{
    QProcess process;
    QString command = "peony";
    QStringList arguments;
    arguments << path;
    process.startDetached(command, arguments);
    process.waitForFinished(-1);
    // 检查进程是否正常退出
    if (process.exitStatus() == QProcess::NormalExit) {
        qDebug() << "peony opened the dir successfully!";
    } else {
        qDebug() << "Failed to open the dir with Pluma!";
    }
}

void ChannelMessage::deleteMeetingFile(QString path)
{
    QFile file(path);
    if (file.remove()) {
        qDebug() << "File removed successfully:" << path;
    } else {
        qDebug() << "Failed to remove file:" << path;
    }
    getMeetingList();
}


void ChannelMessage::sendImageFromJs(const QString &imagePath)
{
    emit sigSendImage(imagePath);
}

void ChannelMessage::selectImage()
{
    emit sigSelectImage();
}

void ChannelMessage::enhanceBlackAndWhite(const QString &imagePath)
{
    qDebug() << "File:" << imagePath;
    emit sigEnhanceBlackAndWhite(imagePath);
}

void ChannelMessage::cutOutImage(const QString &imagePath)
{
    qDebug() << "File:" << imagePath;
    emit sigCutOutImage(imagePath);
}

void ChannelMessage::downloadImage(const QString &base64Image)
{
    emit sigDownloadImage(base64Image);
}

void ChannelMessage::sendDataCollect(const QString &eventCode, const QString &pageName)
{
    emit sigSendDataCollect(eventCode,pageName);
}

void ChannelMessage::openLocation(const QString &imagePath)
{
    if (!QFileInfo::exists(imagePath)) {
        QMessageBox::critical(nullptr, "", tr("No such file or directory!"));
        return;
    }
    QString pathCmd = "\"" + imagePath + "\"";
    QProcess process;
    QString command = "peony";
    QStringList arguments;
    arguments << pathCmd;
    process.startDetached(command, arguments);
    process.waitForFinished(-1);
    // 检查进程是否正常退出
    if (process.exitStatus() == QProcess::NormalExit) {
        qDebug() << "peony opened the file location successfully!";
    } else {
        qDebug() << "Failed to open the file location !";
        QMessageBox::critical(nullptr, "", tr("Failed to open file location!"));

    }
}
