#ifndef CHANNELMESSAGE_H
#define CHANNELMESSAGE_H

#include <QObject>
#include <QDebug>
#include <QJsonArray>

class ChannelMessage : public QObject
{
    Q_OBJECT
public:
    explicit ChannelMessage(QObject *parent = nullptr);
    virtual ~ChannelMessage();
public:
    Q_INVOKABLE void sendMessageFromJs(const QString &jsonMsgHead, const QString &message);
    Q_INVOKABLE void clearMessageFromJs();
    Q_INVOKABLE void getMeetingList();
    Q_INVOKABLE void saveMessageList();
    Q_INVOKABLE void getMessageList();

    Q_INVOKABLE void sendImageFromJs(const QString &imagePath);


signals:
    void sigMessageFromJs(const QString &msgHead, const QString &message);
    void sigClearFromJs();
    void sigUpdateReply(const QString &msgHead, const QString &message);
    void sigStopReply();
    void sigSaveMessageList();
    void sigGetMessageList();
    //一段录音
    void sigStartRecord();
    void sigStopRecord();
    void sigCancelRecord();
    //实时录音
    void sigStartRuntime(QString lan, QString tartget);
    void sigStopRuntime();
    //语音朗读
    void sigStartRead(const QString &message, int index);
    void sigPauseRead(int index);
    void sigContinueRead(int index);
    //语音听写
    void sigClearDictation();
    void sigCopyDictation();
    //会议录音
    void sigStartMeeting();
    void sigPauseMeeting();
    void sigContinueMeeting();
    void sigFinishMeeting();

    void sigRename(QString originalName);
    void sigMeetingList(QString str);

    //图像处理
    void sigSelectImage();
    void sigSendImage(const QString &imagePath);
    void sigEnhanceBlackAndWhite(const QString &imagePath);
    void sigCutOutImage(const QString &imagePath);

    //文生图
    void sigDownloadImage(const QString &base64Image);

    //数据埋点
    void sigSendDataCollect(const QString &eventCode, const QString &pageName);

public slots:
    void updateReply(const QString &jsonMsgHead, const QString &message);
    void stopReply();

    //web接口
    void startVoiceInput();
    void stopVoiceInput();
    void cancelVoiceInput();

    void startRuntime(QString lan, QString tartget);
    void stopRuntime();

    //语音朗读
    void startRead(const QString &msg, QString index);
    void pauseRead(QString index);
    void continueRead( QString index);

    //语音听写
    void clearDictation();
    void copyDictation();

    //会议助手
    void startMeeting();
    void pauseMeeting();
    void continueMeeting();
    void finishMeeting();
    void editName(QString originalName);
    void openMeetingFile(QString path);
    void openMettingFolder(QString path);
    void deleteMeetingFile(QString path);

    //获取会议记录文件生成列表

    //图像处理
    void selectImage();                                  // 选择图片
    void enhanceBlackAndWhite(const QString &imagePath); // 黑白增强
    void cutOutImage(const QString &imagePath);          // 智能抠图
    void openLocation(const QString &imagePath);           // 所在位置

    //文生图
    void downloadImage(const QString &base64Image);      // 下载图片


    //数据埋点
    void sendDataCollect(const QString &eventCode, const QString &pageName);

};

#endif // CHANNELMESSAGE_H
