#ifndef CHATOPERATOR_H
#define CHATOPERATOR_H

#include <memory>
#include <QObject>

#include <kylin-ai/ai-base/nlp.h>
#include <kylin-ai/ai-base/modelconfig.h>
#include <QDateTime>

enum TartgetType{
    RECVMSG = 0,
    SUMMARY,
    CALENDAR
};

class ChatOperator : public QObject
{
    Q_OBJECT
public:
    explicit ChatOperator(QObject *parent = nullptr);
    ~ChatOperator();

    bool initSession();
    bool reinitSession();
    void closeSession();
    void* getSession();
    void clearSession();
    void stopChat();
    bool sendMessage(const QString &message);
    TartgetType target = RECVMSG;
    bool soundFlag = false;

    void notifyUI(const QString &message, bool is_end, int index, int errorCode);
    void notifySummary(const QString &message, bool is_end);

    void setActor(int actorId);

public:
    bool is_end;
    QString result_msg;
    int result_index;

signals:
    void reply(const QString &message, bool is_end, int index, int errorCode);
    void sigSessionCompleted();
    void sigGenerateSummary(const QString &message, bool is_end, int index, int errorCode);
    void sigAddToSchedule(const QString &message, bool is_end, int index, int errorCode);
    void sigSoundMsg(const QString &message, bool is_end, int index);

private:
    class Private;
    std::shared_ptr<Private> d;


    bool initModelVendor();


};

#endif // CHATOPERATOR_H
