#include "ctrl.h"

#include <QMetaType>
#include <QDBusMetaType>
#include <QCoreApplication>
#include <QRegularExpression>
#include <QRegularExpressionMatch>
#include <QDateTime>
#include "usdbrightness.h"

#include <KF5/KScreen/kscreen/output.h>
#include <KF5/KScreen/kscreen/edid.h>
#include <KF5/KScreen/kscreen/mode.h>
#include <KF5/KScreen/kscreen/config.h>
#include <KF5/KScreen/kscreen/getconfigoperation.h>
#include <KF5/KScreen/kscreen/configmonitor.h>
#include <KF5/KScreen/kscreen/setconfigoperation.h>
#include <KF5/KScreen/kscreen/edid.h>
#include <KF5/KScreen/kscreen/types.h>

ctrl::ctrl() : QObject()
{
    if (QGSettings::isSchemaInstalled(QByteArray("org.ukui.control-center.panel.plugins"))) {
        QGSettings gsettings(QByteArray("org.ukui.control-center.panel.plugins"), QByteArray(), this);
//        gsettings.set("gammaforbrightness", true);
    }
    QObject::connect(new KScreen::GetConfigOperation(), &KScreen::GetConfigOperation::finished,
                     [&](KScreen::ConfigOperation *op) {
                         const KScreen::ConfigPtr config = qobject_cast<KScreen::GetConfigOperation *>(op)->config();
                         KScreen::ConfigMonitor::instance()->addConfig(config);
                         for (KScreen::OutputPtr output : config.data()->connectedOutputs()) {
                             qInfo() << "Starting config screen name: " << output->name();
                             m_screenNameList.append(output->name());
                         }
                     });
}

void ctrl::controlMprisPlayerDbus(const QString funcName)
{
    if (m_dbusMprisInterface == nullptr) {
        m_dbusMprisInterface = new QDBusInterface(SESSION_DBUS_NAME_STR_FIELD, SESSION_DBUS_PLAYER_PATH,
                                                  SESSION_DBUS_PLAYER_INTERFACE, QDBusConnection::sessionBus());
    }
    if (m_dbusMprisInterface->isValid()) {
        if (funcName.startsWith("seek")) {
            QStringList itemList = funcName.trimmed().split(':');
            bool ok;
            qlonglong value = itemList.value(1).toLongLong(&ok);
            if (ok) {
                m_dbusMprisInterface->asyncCall("operation", itemList.value(0), QVariantList() << value);
            }
            return;
        }
        if (funcName.startsWith("setRate")) {
            QStringList itemList = funcName.trimmed().split(':');
            bool ok;
            double value = itemList.value(1).toDouble(&ok);
            if (ok) {
                m_dbusMprisInterface->asyncCall("operation", itemList.value(0), QVariantList() << value);
            }
            return;
        }
        m_dbusMprisInterface->asyncCall("operation", funcName, QVariantList());
    }
}

void ctrl::setBrightness(bool isUp)
{
    if (m_dbusBrightnessInterface == nullptr) {
        m_dbusBrightnessInterface = new QDBusInterface(SESSION_DBUS_NAME_BRIGHTNESS, SESSION_DBUS_BRIGHTNESS_PATH,
                                                       SESSION_DBUS_BRIGHTNESS_INTERFACE, QDBusConnection::sessionBus());
    }
    QDBusMessage res =m_dbusBrightnessInterface->call("getPrimaryBrightness");
    int value = res.arguments().first().toUInt();
    qInfo() << "get screen brightness: " << value;
    if (isUp) {
        if (value >= 100) {
            value = 100;
        } else {
            value += 10;
        }
    } else {
        if (value <= 0) {
            value = 0;
        } else {
            value -= 10;
        }
    }
    qInfo() << "set screen brightness: " << value;
    res = m_dbusBrightnessInterface->call("setPrimaryBrightness", (quint32)value);
}

void ctrl::setDefaultAdapterSwitchStatus(bool power)
{
    QDBusMessage dbusMsg = QDBusMessage::createMethodCall(SYSTEM_DBUS_BLE_SERVICE, SYSTEM_DBUS_BLE_PATH,
                                                          SYSTEM_DBUS_BLE_INTERFACE, "setDefaultAdapterAttr");
    QMap<QString, QVariant> adpAttr;
    adpAttr.insert("Powered", QVariant(power));
    dbusMsg << adpAttr;
    QDBusMessage response = QDBusConnection::systemBus().call(dbusMsg, QDBus::NoBlock);

    if (response.type() == QDBusMessage::ReplyMessage) {
        qInfo() << "set Ble status:" << response.arguments().takeFirst().toBool();
    }
}

bool ctrl::addToSchedule(QString summary, QString msg)
{
    qDebug() << summary;
    if (m_scheduleInterface == nullptr) {
        m_scheduleInterface = new QDBusInterface(SESSION_DBUS_NAME_CALENDAR, SESSION_DBUS_PATH_CALENDAR,
                                                  SESSION_DBUS_INTERFACE_CALENDAR, QDBusConnection::sessionBus());
    }

    // 如果字符串不包含年份，则替换为当前年份
    QRegularExpression regex("\\bXXXX\\b");
    QString currentYear = QString::number(QDateTime::currentDateTime().date().year());

    summary.replace(regex, currentYear);

    // 匹配开始时间
    QRegularExpression regex_start_time("开始时间：(\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2})");
    QRegularExpressionMatch match_start_time = regex_start_time.match(summary);

    if (!match_start_time.hasMatch()) {
        qDebug() << "未找到开始时间信息";
        return false;
    }

    QString start_time_str = match_start_time.captured(1);
    QDateTime start_time = QDateTime::fromString(start_time_str, "yyyy-MM-dd HH:mm:ss");

    if (!start_time.isValid()) {
        qDebug() << "无效的开始时间格式：" << start_time_str;
        return false;
    }

    int start_month = start_time.date().month();
    int start_day = start_time.date().day();
    int start_hour = start_time.time().hour();
    int start_minute = start_time.time().minute();
    QString period = (start_hour < 12) ? "上午" : "下午";

    qDebug() << "月：" << start_month << "日：" << start_day << "时：" << start_hour << "分：" << start_minute;
    qDebug() << "时间段：" << period;

    // 匹配结束时间
    QRegularExpression regex_end_time("结束时间：(\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2})");
    QRegularExpressionMatch match_end_time = regex_end_time.match(summary);

    QDateTime end_time;
    if (match_end_time.hasMatch()) {
        QString end_time_str = match_end_time.captured(1);
        end_time = QDateTime::fromString(end_time_str, "yyyy-MM-dd HH:mm:ss");

        if (!end_time.isValid()) {
            qDebug() << "无效的结束时间格式：" << end_time_str;
            end_time = start_time;
        }
    } else {
        qDebug() << "未找到结束时间信息";
        end_time = start_time;
    }

    int end_month = end_time.date().month();
    int end_day = end_time.date().day();
    int end_hour = end_time.time().hour();
    int end_minute = end_time.time().minute();

    qDebug() << "月：" << end_month << "日：" << end_day << "时：" << end_hour << "分：" << end_minute;

//    QRegularExpression regex_topic("主题：(.*)");
//    QRegularExpressionMatch match_str = regex_topic.match(summary);

//    QString topic;

//    if (match_str.hasMatch()) {
//            QString match_topic = match_str.captured(1);
//            topic = match_topic.mid(0,match_topic.indexOf('\\'));
//            topic.replace("\r", "");
//    } else {
//        qDebug() << "未找到主题信息";
//        topic = "预定的会议";  // 默认的会议主题
//    }

    qDebug() << msg;
    QString markId = getCurrentMarkId();

    // 调用DBus接口发送日程安排
    QDBusMessage res = m_scheduleInterface->call("ScheduleNotification",
        start_hour, start_minute, start_day, start_month, period,
        end_minute, end_hour, msg);

    return true;
}

// 获取当前系统时间为标识符
QString ctrl::getCurrentMarkId() {
    QDateTime currentDateTime = QDateTime::currentDateTime();
    return currentDateTime.toString("yyyyMMddhhmmss");
}
