#include "msgpane.h"

#include <QThread>
#include <QVariant>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QKeyEvent>
#include <QWebEngineSettings>
#include <QFileDialog>
#include <QFileSystemWatcher>
#include <QStandardPaths>
#include <ukuistylehelper/ukuistylehelper.h>
#include <QMessageBox>
#include <QMap>
#include <ukui-log4qt.h>

#include "chatoperator.h"
#include "cpushbuttonex.h"
#include "global/kaglobal.h"
#include "tipwidget.h"
#include "renamedialog.h"
#include "kylin-ai/config.h"
//#include "matting-plugin.h"
//#include "scanner-plugin.h"
//#include <photoviewerplugin.h>



//与SDK接口角色对齐
QMap<QString, int> actorLable ={
    {"中英文翻译",1},
    {"英中文翻译",2},
    {"文本扩写",11},
    {"文本润色",12},
    {"内容质检",13},
    {"邮件回复",14},
    {"总结概况",15},
    {"工作总结",16},
    {"代码生成",17},
    {"新建日程",18}
};

CMsgPane::CMsgPane(QWidget *parent) : QWidget(parent)
{
    createAllChildWnd();
    initCtrl();
    initSolts();
    relayout();

    bubble_show_count = 0;
    bubble_show_speech_count = 0;
    initChatOperator();
    initBubbleUpdateTimer();
    initBubbleUpdateSpeechTimer();
}

void CMsgPane::createAllChildWnd()
{
#define NEW_OBJECT(pObj, TYPE) \
    if (NULL == pObj) { pObj = new TYPE(this); }

//    NEW_OBJECT(m_pViewChat, QWebEngineView);
    NEW_OBJECT(m_pViewChat, KWebEngineView);
    NEW_OBJECT(m_channel, QWebChannel);
    NEW_OBJECT(m_channelMsg, ChannelMessage);
    NEW_OBJECT(m_btnVoice, CPushButtonEx);
    NEW_OBJECT(m_labSeparatorLine, QLabel);
    NEW_OBJECT(m_reader, Reader);
    NEW_OBJECT(m_recorder, Recorder);
    NEW_OBJECT(m_runtimeRecorder, RuntimeRecorder);
    NEW_OBJECT(m_meetingRecorder, MeetingRecorder);
    m_AIDrawing = new AITextGeneratedPic();
//    NEW_OBJECT(m_aiMatting, Matting);
//    NEW_OBJECT(m_aiScannerPlugin, ScannerPlugin);
//    NEW_OBJECT(m_photoViewerPlugin, PhotoViewerPlugin);
}

void CMsgPane::initCtrl()
{
    // 按照设计稿设计，语音按钮大小为 16x16
    m_btnVoice->setFixedSize(16, 16);
    m_btnVoice->setIcon(QIcon(QString(":/icon/icon/voice.svg")));
    m_btnVoice->hide();

    m_labSeparatorLine->setFixedHeight(1);
    m_labSeparatorLine->setStyleSheet("background-color:#4A7ABA");
    m_labSeparatorLine->hide();

    m_pViewChat->page()->setWebChannel(m_channel);
    m_channel->registerObject(QStringLiteral("channelMsg"), m_channelMsg);

    m_pViewChat->setContextMenuPolicy(Qt::NoContextMenu);
    m_pViewChat->load(QUrl("qrc:/html/html/index1.html"));
//    m_pViewChat->load(QUrl("qrc:/html/html/meeting-detail.html"));
    m_pViewChat->show();
    m_pViewChat->setFixedWidth(402);
    connect(m_pViewChat, &QWebEngineView::loadFinished, this, &CMsgPane::OnWebViewLoaded);
    connect(m_pViewChat, &QWebEngineView::urlChanged, this, &CMsgPane::onUrlChanged);

    // 按照设计稿设计，聊天界面大小为 402x568
//    this->setFixedSize(402, 568);
}

void CMsgPane::initSolts()
{
    connect(m_reader, &Reader::sigReadEnd, this, &CMsgPane::onReadEnd);
    connect(m_recorder, &Recorder::sigSendMsg, this, &CMsgPane::onSendSpeechMsg);
    connect(m_recorder,&Recorder::reply,this,&CMsgPane::onRecvMsg);
    connect(m_recorder,&Recorder::sigCancelSendVoice,this,&CMsgPane::onCancelAudio);

    connect(m_runtimeRecorder, &RuntimeRecorder::sigShowText, this, &CMsgPane::onShowText);
    connect(m_meetingRecorder, &MeetingRecorder::sigShowSpeech, this, &CMsgPane::onShowSpeech);
    connect(m_meetingRecorder, &MeetingRecorder::sigAppendSpeech, this, &CMsgPane::onAppendSpeech);


    connect(g_gsettings, &QGSettings::changed, [this](QString key){
        bool flag ;
        //更改模型之后，重新初始化session
        if (key == "isNlpSetup")
        {
            m_chatOperator->closeSession();
        }else if(key =="isSpeechSetup"){
            flag = g_gsettings->get("isNlpSetup").toBool();
            if (flag){

                m_meetingRecorder->reinitSession();
                m_reader->reinitSession();
            }
            else{
                m_meetingRecorder->closeSession();
                m_reader->closeSession();
            }
            //语音对话在点击说话时initSession
            m_recorder->closeSession();
        }

    });

    //文本对话
    connect(m_channelMsg, &ChannelMessage::sigMessageFromJs, this, &CMsgPane::onSendMsg);
    connect(m_channelMsg, &ChannelMessage::sigClearFromJs, this, &CMsgPane::onClearMsg);
    connect(m_channelMsg, &ChannelMessage::sigUpdateReply, this, &CMsgPane::onSendMsg);
    connect(m_channelMsg, &ChannelMessage::sigStopReply, this, &CMsgPane::onStopReply);
    connect(m_channelMsg, &ChannelMessage::sigSaveMessageList, this, &CMsgPane::onSaveMessageList);
    connect(m_channelMsg, &ChannelMessage::sigGetMessageList, this, &CMsgPane::onLoadMessageList);
    //录音功能
    connect(m_channelMsg, &ChannelMessage::sigStartRecord, this, &CMsgPane::onSendAudio);
    connect(m_channelMsg, &ChannelMessage::sigStopRecord, this, &CMsgPane::onStopAudio);
    connect(m_channelMsg, &ChannelMessage::sigCancelRecord, this, &CMsgPane::onCancelAudio);
    connect(m_channelMsg, &ChannelMessage::sigStartRuntime, this, &CMsgPane::onSendRuntimeAudio);
    connect(m_channelMsg, &ChannelMessage::sigStopRuntime, this, &CMsgPane::onStopRuntimeAudio);
    //语音朗读
    connect(m_channelMsg, &ChannelMessage::sigStartRead, this, &CMsgPane::onStartRead);
    connect(m_channelMsg, &ChannelMessage::sigPauseRead, this, &CMsgPane::onPauseRead);
    connect(m_channelMsg, &ChannelMessage::sigContinueRead, this, &CMsgPane::onContinueRead);
    //语音听写
    connect(m_channelMsg, &ChannelMessage::sigClearDictation, this, &CMsgPane::onClearDictation);
    connect(m_channelMsg, &ChannelMessage::sigCopyDictation, this, &CMsgPane::onCopyDictation);
    //会议助手
    connect(m_channelMsg, &ChannelMessage::sigStartMeeting, this, &CMsgPane::onSendMeetingAudio);
    connect(m_channelMsg, &ChannelMessage::sigPauseMeeting, this, &CMsgPane::onPauseMeetingAudio);
    connect(m_channelMsg, &ChannelMessage::sigContinueMeeting, this, &CMsgPane::onContinueMeetingAudio);
    connect(m_channelMsg, &ChannelMessage::sigFinishMeeting, this, &CMsgPane::onStopMeetingAudio);

    connect(m_channelMsg, &ChannelMessage::sigRename, this, &CMsgPane::onRename);
    connect(m_channelMsg, &ChannelMessage::sigMeetingList, this, &CMsgPane::onMeetingList);
    //图像处理
    connect(m_channelMsg, &ChannelMessage::sigSelectImage, this, &CMsgPane::onSelectImage);
    connect(m_pViewChat, &KWebEngineView::sigShowImage, this, &CMsgPane::onShowImage);
    connect(m_channelMsg, &ChannelMessage::sigSendImage, this, &CMsgPane::onSendImage);
    connect(m_channelMsg, &ChannelMessage::sigEnhanceBlackAndWhite, this, &CMsgPane::onEnhanceBlackAndWhite);
    connect(m_channelMsg, &ChannelMessage::sigCutOutImage, this, &CMsgPane::onCutOutImage);
    //文生图
    connect(m_AIDrawing, &AITextGeneratedPic::pictureUpdateSignal, this,&CMsgPane::updateToShowAIPic);
    connect(m_channelMsg, &ChannelMessage::sigDownloadImage, this, &CMsgPane::onDownloadImage);

    //数据埋点
    connect(m_channelMsg,&ChannelMessage::sigSendDataCollect,this,&CMsgPane::onSendDataCollect);

    qint64 uid = static_cast<qint64>(getuid());
    QDBusInterface iface("org.freedesktop.Accounts",
                         "/org/freedesktop/Accounts",
                         "org.freedesktop.Accounts",
                         QDBusConnection::systemBus());
    QDBusReply<QDBusObjectPath>objPath = iface.call("FindUserById", uid);
    QDBusConnection::systemBus().connect("org.freedesktop.Accounts",
                                         objPath.value().path(),
                                         "org.freedesktop.Accounts.User",
                                         QString("Changed"), this, SLOT(userAccountsChanged()));
}

void CMsgPane::initChatOperator()
{
    m_chatOperator = new ChatOperator;
    connect(m_chatOperator, &ChatOperator::reply, this, &CMsgPane::onRecvMsg);
    connect(m_chatOperator, &ChatOperator::sigGenerateSummary, this, &CMsgPane::onGenerateSummary);
    connect(m_chatOperator, &ChatOperator::sigAddToSchedule, this, &CMsgPane::onAddToSchedule);
    connect(m_chatOperator, &ChatOperator::sigSoundMsg, [this](const QString &message, bool is_end, int index){
            m_reader->textToSoundOutput(message, is_end, index);
    });

}

void CMsgPane::initBubbleUpdateTimer()
{
    bubble_update_timer = new QTimer;
    bubble_update_timer->setInterval(40);
    connect(bubble_update_timer, &QTimer::timeout, this, &CMsgPane::updateBubble);
}

void CMsgPane::relayout()
{
    QVBoxLayout *layoutRMain = new QVBoxLayout();;
    layoutRMain->addWidget(m_pViewChat);
    layoutRMain->addWidget(m_labSeparatorLine);
    layoutRMain->setSpacing(0);
    layoutRMain->setMargin(0);

    setLayout(layoutRMain);
}

void CMsgPane::onBackBtnClicked()
{
    qDebug() << "当前页面"<< m_pViewChat->url();
    if(m_pViewChat->url() == QUrl("qrc:/html/html/meeting-recording.html"))
    {
        QString js_str = QString("clickBack()");
        m_pViewChat->page()->runJavaScript(js_str);
    } else {
        m_pViewChat->load(QUrl("qrc:/html/html/index1.html"));
        //加载主页面聊天记录
        onLoadMessageList();
    }
}

void CMsgPane::onUrlChanged(const QUrl &url)
{
    emit sigUrlChanged(url);
}

void CMsgPane::onClearMsg()
{
    m_chatOperator->clearSession();
    QString jsStr1 = QString("contextCleared()");
    m_pViewChat->page()->runJavaScript(jsStr1);

}

//更换信号监听，用于设置语音部分调用；onSendMsg  connect
void CMsgPane::onSoundMsg(const QString &msgHead, const QString &msg)
{
//    m_chatOperator->soundFlag = true;
    onSendMsg(msgHead, msg);
}


void CMsgPane::onSendMsg(const QString &jsonMsgHead, const QString &msg)
{ 
    //解析jsonMsgHead
    QJsonDocument jsonDoc = QJsonDocument::fromJson(jsonMsgHead.toUtf8());
    if (!jsonDoc.isNull()) {
        if (jsonDoc.isObject()) {
            jsonObjToQt = jsonDoc.object();
            msgHead = jsonObjToQt["msgHead"].toString();
            updateReplyFlag= jsonObjToQt["updateReplyFlag"].toBool();
            speechFlag = jsonObjToQt["speechFlagMsg"].toBool();
        }
    }

    send_message = msg;

    m_chatOperator->target = RECVMSG;
    new_reply = true;
    bool flag = false;   //识别是否为系统控制命令

    //1、判断是否为dbus命令
    if (StartUpTools::getInstance()->inspectCtrl(msg)) {
        QTime dieTime = QTime::currentTime().addMSecs(800);
        while (QTime::currentTime() < dieTime) {
            QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
        }
        qInfo() << "The wake word was matched successfully!";
        flag = true;
    }
    QString key = StartUpTools::getInstance()->findNmKey(msg);
    if (!key.isEmpty()) {
        QTime dieTime = QTime::currentTime().addMSecs(800);
        while (QTime::currentTime() < dieTime) {
            QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
        }
        qInfo() << "The wake word was matched successfully!";
        StartUpTools::getInstance()->startUp(key);
        flag = true;
    }
    //2、判断打开还是关闭操作
    opType op = StartUpTools::getInstance()->isExit(msg);
    if (op != ORTHER){
        QString target = msg.mid(2);
        key = StartUpTools::getInstance()->findKey(target, op);

        //3、判断是否为可支持应用
        if (!key.isEmpty()) {
            QTime dieTime = QTime::currentTime().addMSecs(800);
            while (QTime::currentTime() < dieTime) {
                QCoreApplication::processEvents(QEventLoop::AllEvents, 100);
            }
            qInfo() << "The wake word was matched successfully!";
            if (op == CLOSE) {
                StartUpTools::getInstance()->killProcess(key);
            } else if (op == OPEN){
                StartUpTools::getInstance()->startUp(key);
            }
            flag = true;
        }
    }

    //判断是否有标签模板
    if(msgHead != ""){
        int actorId = actorLable[msgHead];

        if(m_chatOperator->getSession() == nullptr){
            m_chatOperator->initSession();
        }
        m_chatOperator->setActor(actorId);
        if(actorId == 18)
            m_chatOperator->target = TartgetType::CALENDAR;
    }
    //AI助手生成
    jsonObjToWeb["msgHead"] = msgHead;
    jsonObjToWeb["modelMsg"] = "ai";
    jsonObjToWeb["colorMsg"] = "";
    QJsonDocument aiJsonSendWebDoc(jsonObjToWeb);
    QString jsonMsgHeadToWeb = aiJsonSendWebDoc.toJson(QJsonDocument::Indented);

    if (!updateReplyFlag && !speechFlag) {
        pushMsg("addMsg", jsonMsgHeadToWeb, msg);
    }

    // AI文生图接口
    bool ai_res = false;
//    if (msgHead.isEmpty())
        ai_res = m_AIDrawing->getWhetherToDrawForAI(msg);
//    else if(msgHead.compare("图片生成") == 0){
//        ai_res = true;
//    }
//    else
//        ai_res = false;

    if (ai_res){
        jsonObjToWeb["drawMsg"] = true;
        QJsonDocument aiJsonSendWebDoc(jsonObjToWeb);
        QString jsonMsgHeadToWeb = aiJsonSendWebDoc.toJson(QJsonDocument::Indented);
        m_AIDrawing->wordInformationProcessing(msg);
        pushMsg("addRecvMsg", jsonMsgHeadToWeb, "好的，正在为您生成图片：");
        QString jsStr1 = QString("isReplyEnd(true)");
        m_pViewChat->page()->runJavaScript(jsStr1);
        jsonObjToWeb["drawMsg"] = "";

        return;
    }

    //判断是否是空消息
    if(msg.length() == 0){
        jsonObjToWeb["colorMsg"] = "red";
        QJsonDocument aiJsonSendWebDoc(jsonObjToWeb);
        QString jsonMsgHeadToWeb = aiJsonSendWebDoc.toJson(QJsonDocument::Indented);
        pushMsg("addRecvMsg", jsonMsgHeadToWeb, "未识别到说话。请靠近话筒，并确保在安静的环境中清晰地说话。");
        QString jsStr1 = QString("isReplyEnd(true)");
        m_pViewChat->page()->runJavaScript(jsStr1);
        jsonObjToWeb["colorMsg"] = "";
        return;
    }

    if (flag){
        pushMsg("addRecvMsg", jsonMsgHeadToWeb, "好的，已为您执行操作");
        QString jsStr1 = QString("isReplyEnd(true)");
        m_pViewChat->page()->runJavaScript(jsStr1);
        return;
    }
    m_chatOperator->sendMessage(msg);
    QString js_str2 = QString("setReplyStale()"); //设置上一条回答不可重新生成
    m_pViewChat->page()->runJavaScript(js_str2);
}
void CMsgPane::onStopReply()
{
    new_reply = true;
    recv_message ="";
    bubble_show_count = 0;
    bubble_update_timer->stop();
    speech_bubble_update_timer->stop();
    QString jsStr1 = QString("isReplyEnd(true)");
    m_pViewChat->page()->runJavaScript(jsStr1);
    m_chatOperator->stopChat();
}

void CMsgPane::OnWebViewLoaded()
{
    QTimer::singleShot(1000, this, [this](){
#ifdef QT_DEBUG
    // only for debug
//        pushMsg("addMsg", "小麒小麒，谁是世界上最好的操作系统？");
//        pushMsg("addRecvMsg", "关于哪个操作系统是最好的，这是一个非常主观的问题，取决于个人偏好、需求和使用情境。以下是一些常见的操作系统，每个都有其优点和适用场景。");
//        pushMsg("cutoffMsg", "-------上下文关联已清除-------");
//        pushMsg("addMsg", "小麒小麒，谁是世界上最好的操作系统？");
//        pushMsg("addRecvMsg", "关于哪个操作系统是最好的，这是一个非常主观的问题，取决于个人偏好、需求和使用情境。以下是一些常见的操作系统，每个都有其优点和适用场景。");
//        pushMsg("flush", "0");
#endif
    });

    initCurrentUserName();
    initCurrentUserIcon();
}

void CMsgPane::onRecvMsg(QString msg, bool is_end, int index, int errorCode)
{
    //NLP错误码
    if(errorCode != 0)
    {
        jsonObjToWeb["modelMsg"] = "ai";
        jsonObjToWeb["colorMsg"] = "red";
        QJsonDocument aiJsonSendWebDoc(jsonObjToWeb);
        QString jsonMsgHeadToWeb = aiJsonSendWebDoc.toJson(QJsonDocument::Indented);
        pushMsg("addRecvMsg", jsonMsgHeadToWeb, msg);
        QString jsStr1 = QString("isReplyEnd(true)");
        m_pViewChat->page()->runJavaScript(jsStr1);
        jsonObjToWeb["modelMsg"] = "";
        jsonObjToWeb["colorMsg"] = "";
        return;
    }

    if (new_reply)
    {
        pushMsg("addRecvMsg", "", "");
    }
    recv_message.append(msg);
    new_reply = is_end;
    if (!bubble_update_timer->isActive())
        bubble_update_timer->start();

    QString jsStr1 = QString("isReplyEnd(false)");
    m_pViewChat->page()->runJavaScript(jsStr1);
}

void CMsgPane::onGenerateSummary(QString msg, bool is_end, int index)
{
    if (index == 0){
        m_summary="";
    }
    msg.replace("\\", "\\\\");
    msg.replace("\n", "\\n");
    msg.replace("\"", "\\\"");
    m_summary.append(msg);
    if(is_end){
        saveMeetingToFile();
        m_chatOperator->clearSession();
    }
}

void CMsgPane::onShowText(const QString text)
{
    pushMsg("showText", "", text);
}

void CMsgPane::onShowSpeech(const QString text)
{
//    QString msg = this->m_finishedSpeech + text;
//    msg.replace('\n', "");
//    QString js_str = QString("updateMsg('%1')").arg(msg);
//    qDebug() << js_str;
//    m_pViewChat->page()->runJavaScript(js_str);
}

void CMsgPane::onAppendSpeech(const int speakerID, const QString text)
{
//    this->m_finishedSpeech += text;
    QString js_str = QString("pushMsg('%1','%2')").arg(speakerID).arg(text);
    qDebug() << js_str;
    m_pViewChat->page()->runJavaScript(js_str);
}

/**
 * @brief CMsgPane::updateToShowAIPic  AI文生图返回图片函数
 * @param imageData 图像数据
 * @param errorMessage 错误信息
 */
void CMsgPane::updateToShowAIPic(VisionImageData *imageData, QString errorMessage)
{
    if (imageData != nullptr) {
        qDebug()<<"图片base64 data数据："<<imageData->data
                <<"图片字节数："<<imageData->data_size
                <<"图像宽高："<<imageData->width<<imageData->height;

        //上传图像到html，并更新当前的对话框
        QPixmap map;
        map.loadFromData(imageData->data,imageData->data_size);

        QImage image = map.toImage();
       // 将QImage转换为QByteArray
       QByteArray byteArray;
       QBuffer buffer(&byteArray);
       buffer.open(QIODevice::WriteOnly);
       image.save(&buffer, "PNG");
       buffer.close();
       QString str = QString("data:image/png;base64," + byteArray.toBase64());
        m_pViewChat->page()->runJavaScript(QString("insertImage('%1')").arg(str));
    }
    if (!errorMessage.isEmpty()){
        qDebug()<<"图片生成失败，imageData为空，错误信息(存在多种错误反馈)： "<<errorMessage;
        pushMsg("updateMsg", "", errorMessage + "，请重新尝试。");
    }
    QString jsStr2 = QString("isReplyEnd(true)");
    m_pViewChat->page()->runJavaScript(jsStr2);
}

/**
 * @brief CMsgPane::updateBubble 按字去更新内容
 */
void CMsgPane::updateBubble()
{
    bool loading = bubble_show_count == recv_message.length();
    if (new_reply && loading)
    {
        recv_message = "";
        bubble_show_count = 0;
        bubble_update_timer->stop();

        QString jsStr = QString("isReplyEnd(true)");
        m_pViewChat->page()->runJavaScript(jsStr);
        return;
    }
    QString show_msg = recv_message.left(loading ? bubble_show_count : ++bubble_show_count);
    if (!loading)
        loading_dot_index = 0;
    QString dot_dot = "...";
    pushMsg("updateMsg", "", show_msg.append(dot_dot.left(loading ? loading_dot_index++ / 10 % 4 : 0)));
}

void CMsgPane::pushMsg(QString func, QString jsonMsgHeadToWeb, QString msg)
{
    jsonMsgHeadToWeb.replace('\n', "");
    jsonMsgHeadToWeb.replace("\\", "\\\\");
    jsonMsgHeadToWeb.replace('\n', "\\n");
    jsonMsgHeadToWeb.replace("\r", "\\r");
    jsonMsgHeadToWeb.replace("\"", "\\\"");

    msg.replace("\\", "\\\\");
    msg.replace("\n", "\\n");
    msg.replace("\r", "\\r");
    msg.replace("\"", "\\\"");

    if(func == "addMsg" || func == "addRecvMsg"){
        QString js_str = QString("%1('%2','%3')").arg(func).arg(jsonMsgHeadToWeb).arg(msg);
//        qDebug()<<"js_str"<<js_str;
        m_pViewChat->page()->runJavaScript(js_str);
        QString js_str2 = QString("setReplyStale()"); //设置上一条回答不可重新生成
        m_pViewChat->page()->runJavaScript(js_str2);
    }else{
        QString js_str = QString("%1(\"%2\")").arg(func).arg(msg);
        m_pViewChat->page()->runJavaScript(js_str);
    }
}

void CMsgPane::initCurrentUserName()
{
    QString objpath = QString("/org/freedesktop/Accounts/User%1").arg(getuid());

    QString userName;
    QDBusInterface *iproperty = new QDBusInterface("org.freedesktop.Accounts",
                                                   objpath,
                                                   "org.freedesktop.DBus.Properties",
                                                   QDBusConnection::systemBus());
    QDBusReply<QMap<QString, QVariant> > ipropertyReply = iproperty->call("GetAll", "org.freedesktop.Accounts.User");
    if (ipropertyReply.isValid()){
        QMap<QString, QVariant> propertyMap = ipropertyReply.value();

        userName = propertyMap.find("RealName").value().toString();
    }

    QString js_str = QString("%1(\"%2\")").arg("setUser").arg(userName);
    m_pViewChat->page()->runJavaScript(js_str);

    return;
}

void CMsgPane::initCurrentUserIcon()
{

    QString objpath = QString("/org/freedesktop/Accounts/User%1").arg(getuid());

    QString iconfile;
    QDBusInterface *iproperty = new QDBusInterface("org.freedesktop.Accounts",
                                                   objpath,
                                                   "org.freedesktop.DBus.Properties",
                                                   QDBusConnection::systemBus());
    QDBusReply<QMap<QString, QVariant> > ipropertyReply = iproperty->call("GetAll", "org.freedesktop.Accounts.User");
    if (ipropertyReply.isValid()){
        QMap<QString, QVariant> propertyMap = ipropertyReply.value();

        iconfile = propertyMap.find("IconFile").value().toString();
    }

    QImage image(iconfile);

    QByteArray ba;
    QBuffer buf(&ba);
    image.save(&buf, "png");
    buf.close();
    QString base64Str = QString::fromLocal8Bit(ba.toBase64());
    QString base64StrIcon = "data:image/png;base64," + base64Str;

    QString js_str = QString("%1(\"%2\")").arg("setUserIcon").arg(base64StrIcon);
    m_pViewChat->page()->runJavaScript(js_str);

    return;
}

void CMsgPane::userAccountsChanged()
{
    initCurrentUserName();
    initCurrentUserIcon();

    return;
}

void CMsgPane::saveDialogToFile(const QString &dialogJsonString)
{
    // 获取当前时间
    QDateTime currentTime = QDateTime::currentDateTime();

    // 格式化当前时间为字符串，包含年、月、日、小时、分钟和秒
    QString formattedTime = currentTime.toString("yyyyMMdd_hhmmss");

    QString documentsDir = QDir::homePath() + "/文档";

    // 使用格式化后的时间作为文件名
    QString fileName = documentsDir + "/kylin-aiassistant/dialog/dialog_" + formattedTime + ".json";

    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        return;
    }

    file.write(dialogJsonString.toUtf8());

    file.close();

    // 由于CMsgPane::saveDialogToFile是被异步调用的，所以这里通过信号通知历史会话界面当前已经保存了会话到文件，以此更新历史会话界面。
    emit sigSaveDialog(fileName);

    return;
}

void CMsgPane::saveMessageToFile(const QString &msgJsonString)
{
    QString documentsDir = QDir::homePath() + "/文档";

    QString fileName = documentsDir + "/kylin-aiassistant/dialog/message.json";

    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) {
        return;
    }

    file.write(msgJsonString.toUtf8());

    file.close();

    return;
}

void CMsgPane::saveMeetingToFile()
{
    // 获取当前时间
    QDateTime currentTime = QDateTime::currentDateTime();

    // 格式化当前时间为字符串，包含年、月、日、小时、分钟和秒
    QString formattedTime = currentTime.toString("yyyyMMdd_hhmmss");

    QString documentsDir = QDir::homePath() + "/文档";

    // 使用格式化后的时间作为文件名
    QString fileName = documentsDir + "/kylin-aiassistant/meeting/meeting_" + formattedTime + ".txt";

    QFile file(fileName);
    if (!file.open(QIODevice::ReadWrite | QIODevice::Text)) {
        return;
    }
    if(m_origin == EMPTYMETTINGFLAG)
    {
    file.write(("【智能总结】 \n\n【会议原文】\n\n") );
    }
    else
    {
    m_origin.replace(EMPTYMETTINGFLAG,"");
    file.write(("【智能总结】\n" + m_summary + "\n【会议原文】\n" + m_origin).toUtf8());
    }
    //清空存储的内容
    m_summary.clear();
    m_origin.clear();
    file.close();

    //如果有临时文件，应当将当前文件重命名到临时文件名称，将临时文件删除，并刷新界面
    if(!m_tempMettingFileName.isEmpty())
    {
        //临时文件路径
        QString tempFilePath = documentsDir+"/kylin-aiassistant/meeting/"+m_tempMettingFileName;
        QFile tempFile(tempFilePath);
        if(!tempFile.exists())
        {
            qDebug() << tempFilePath;
            return;
        }
        //QString tempFileName = m_tempMettingFileName.left(27);
        //获取文件生成时间并作为文件名称进行重命名
        QFileInfo fileInfo(fileName);
        QString trueFileName = "metting_"+fileInfo.created().toString("yyyyMMdd_hhmmss")+".txt";
        QString trueFilePath = documentsDir+"/kylin-aiassistant/meeting/"+trueFileName;
        bool renameFlag = QFile::rename(fileName,trueFilePath);
        bool removeTempFlag = QFile::remove(tempFilePath);
        if(renameFlag && removeTempFlag)
        {
            qDebug() << "metting file save successful";

            QString jsStr1 = QString("completedSummary('%1')").arg(trueFilePath);
            m_pViewChat->page()->runJavaScript(jsStr1);
        }
        else
        {
            qDebug() << "metting file save failed";
            qDebug() <<  renameFlag << removeTempFlag;
        }
        //刷新一下界面内容
         m_channelMsg->getMeetingList();
    }

}

void CMsgPane::saveMeetingToTempFile()
{
    // 获取当前时间
    QDateTime currentTime = QDateTime::currentDateTime();
    // 格式化当前时间为字符串，包含年、月、日、小时、分钟和秒
    QString formattedTime = currentTime.toString("yyyyMMdd_hhmmss");
    QString documentsDir = QDir::homePath() + "/文档";
    // 使用格式化后的时间作为文件名
    QString fileName = documentsDir + "/kylin-aiassistant/meeting/meeting_" + formattedTime + ".txt.temp";
    //将临时文件名称存入字符串方便后续判断
    m_tempMettingFileName = "meeting_"+formattedTime+".txt.temp";
    QFile file(fileName);
    if (!file.open(QIODevice::ReadWrite | QIODevice::Text)) {
        return;
    }
    file.close();
}

void CMsgPane::saveDialog()
{
    // 先保存当前会话
    m_pViewChat->page()->runJavaScript("getActiveDialog()", [this](const QVariant &v) { this->saveDialogToFile(v.toString());});

    return;
}

void CMsgPane::clearDialog()
{
    // 然后清除当前会话
    m_pViewChat->page()->runJavaScript("clearActiveDialog()");

    return;
}

void CMsgPane::loadPlugin(QString pluginHtmlFile)
{
    m_pViewChat->load(QUrl(pluginHtmlFile));
}

void CMsgPane::onAddToSchedule(QString msg, bool is_end, int index)
{
    if (index == 0){
        m_summary="";
    }
    msg.replace("\\", "\\\\");
    msg.replace("\n", "\\n");
    msg.replace("\"", "\\\"");
    m_summary.append(msg);
    if(is_end){
       bool success = StartUpTools::getInstance()->addToSchedule(m_summary, send_message);
       //AI助手生成
       jsonObjToWeb["msgHead"] = msgHead;
       jsonObjToWeb["modelMsg"] = "ai";
       QJsonDocument aiJsonSendWebDoc(jsonObjToWeb);
       QString scheJsonMsgHeadToWeb = aiJsonSendWebDoc.toJson(QJsonDocument::Indented);
       if(success) {
           pushMsg("addRecvMsg", scheJsonMsgHeadToWeb, "好的，已为您添加日程");
       } else {
           pushMsg("addRecvMsg", scheJsonMsgHeadToWeb, "非常抱歉，我无法为您添加日程，因为您没有提供开始时间等信息，或者您提供的信息我无法识别。为了成功添加日程，请确保您提供了清晰、准确的日程信息。");
       }
       QString jsStr1 = QString("isReplyEnd(true)");
       m_pViewChat->page()->runJavaScript(jsStr1);
   }
}

void CMsgPane::onSendAudio()
{
    m_recorder->startRecord();

}

void CMsgPane::onStopAudio()
{
    m_recorder->stopRecord();
    //录音结束，调用recorder的发送接口，异步返回结果
}

void CMsgPane::onCancelAudio()
{
    m_recorder->cancelRecord();
    QString jsStr = QString("cancelSendVoice()");
    m_pViewChat->page()->runJavaScript(jsStr);

}

void CMsgPane::onSendRuntimeAudio(QString lan, QString target)
{
    m_runtimeRecorder->startRuntimeRecord();
    //开始录音，通过runtimeRecorder实时发送录音数据

    //不应该在此处执行js命令，正确流程是  callback发送信号，触发槽函数，槽函数调用pushMsg。
//    pushMsg("showText", "", "测试听写测试听写测试听写测试听写测试听写测试听写");
}

void CMsgPane::onStopRuntimeAudio()
{
    m_runtimeRecorder->stopRuntimeRecord();
    //停止录音，关闭会话
}

void CMsgPane::onStartRead(const QString &msg, int index)
{
    KyInfo() << "startRead" << msg << index;
    m_reader->sendText(msg, index);
}

void CMsgPane::onPauseRead(int index)
{
    KyInfo() << "pauseRead" << index;
    m_reader->pauseReading(index);

}

void CMsgPane::onContinueRead(int index)
{
    KyInfo() << "continueRead" << index;
    m_reader->continueReading(index);

}

void CMsgPane::onReadEnd(int index)
{
    KyInfo() << "ReadEnd";
    QString js_str = QString("setReadEnd(%1)").arg(index);
    m_pViewChat->page()->runJavaScript(js_str);
    qDebug()<< js_str;

}

void CMsgPane::onClearDictation()
{
    TipWidget::showTip(tr("Cleared"), 1000, QSize(106, 48), parentWidget(), TipWidget:: OK);
}

void CMsgPane::onCopyDictation()
{
    TipWidget::showTip(tr("Copy Successful"), 1000, QSize(120, 48), parentWidget(), TipWidget::OK);
}

void CMsgPane::onSendMeetingAudio()
{
    m_meetingRecorder->startMeetingRecord();
    pushMsg("startTimer","","");
    //todo : 开始录音之后，调用接口发送音频数据，等待callback返回输出

}

void CMsgPane::onPauseMeetingAudio()
{
    m_meetingRecorder->pauseMeetingRecord();
}

void CMsgPane::onContinueMeetingAudio()
{
     m_meetingRecorder->continueMeetingRecord();
}

void CMsgPane::onStopMeetingAudio()
{
    //停止会议记录
    m_meetingRecorder->stopMeetingRecord();
    //保存临时文件，方便显示正在生成中的界面
    saveMeetingToTempFile();
    //解析并显示会议总结
    m_pViewChat->page()->runJavaScript("getMeetingContent()", [this](const QVariant &v) { this->processMessageList(v.toString());});
}

void CMsgPane::onRename(QString originalName)
{
    RenameDialog dialog(originalName, this);
    if (dialog.exec() == QDialog::Accepted) {
        QString newName = dialog.getNewName();
        QString js_str = QString("rename(\"%1\",\"%2\")").arg(originalName).arg(newName);
        m_pViewChat->page()->runJavaScript(js_str);
        qDebug()<< js_str;
    }
}

void CMsgPane::onMeetingList(QString str)
{
    str.replace('\n', "");
    QString js_str = QString("loadMeetingList('%1')").arg(str);
    //qDebug() << js_str;
    m_pViewChat->page()->runJavaScript(js_str);
}

void CMsgPane::processMessageList(QString jsonStr)
{
    m_chatOperator->target = SUMMARY;
    //解析jsonStr，提取content的值，拼接成QString字符串，赋值给m_origin
        QJsonDocument jsonDoc = QJsonDocument::fromJson(jsonStr.toUtf8());
        if (!jsonDoc.isNull()) {
            if (jsonDoc.isArray()){
                QJsonArray jsonArray = jsonDoc.array();
                for (const QJsonValue &value : jsonArray) {
                    if (value.isObject()) {
                        QJsonObject jsonObject = value.toObject();
                        if(jsonObject.contains("name")){
                            QString name = jsonObject.value("name").toString();
                            m_origin.append(name + " ");
                        }
                        if(jsonObject.contains("start") && jsonObject.contains("end")){
                            QString time = jsonObject.value("start").toString() + "-" + jsonObject.value("end").toString();
                            m_origin.append(time + "\n");
                        }
                        if (jsonObject.contains("content")) {
                            QString content = jsonObject.value("content").toString();
                            m_origin.append(content + "\n");
                        }
                    }
                }
            }
        }
     //如果解析出的总结内容中为空，则应当添加空值标志位，方便进行总结和生成时的判断
     if(m_origin.isEmpty())
     {
          m_origin.append(EMPTYMETTINGFLAG);
     }
    //设置会议总结角色，发送会议原文
    m_chatOperator->setActor(15);
    m_chatOperator->sendMessage(m_origin);
}

void CMsgPane::onSaveMessageList()
{
    //跳转前更新保存当前会话内容
    m_pViewChat->page()->runJavaScript("getActiveDialog()", [this](const QVariant &v) { this->saveMessageToFile(v.toString());});
}

void CMsgPane::onLoadMessageList()
{
    QString documentsDir = QDir::homePath() + "/文档";
    QString directoryPath = documentsDir + "/kylin-aiassistant/dialog";
    QDir dir(directoryPath);
    if (!dir.exists()) {
        qDebug() << "目录不存在：" << directoryPath;
        return;
    }
    QString fileName = "message.json";
    QFile file(directoryPath + "/" + fileName);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) {
        qDebug() << "无法打开文件：" << fileName;
        return;
    }

    QByteArray jsonData = file.readAll();
    file.close();

    QJsonParseError parseError;
    QJsonDocument jsonDoc = QJsonDocument::fromJson(jsonData, &parseError);
    if (parseError.error != QJsonParseError::NoError) {
        qDebug() << "JSON 解析错误：" << parseError.errorString();
        return;
    }

    QJsonArray jsonArray = jsonDoc.array();
    QString str = QJsonDocument(jsonArray).toJson(QJsonDocument::Indented);
    str.replace("\\", "\\\\");
    str.replace("\n", "\\n");
    str.replace("\"", "\\\"");
    QString js_str = QString("loadMessageList('%1')").arg(str);
    m_pViewChat->page()->runJavaScript(js_str);
}

bool CMsgPane::onDownloadImage(const QString &base64Image)
{
    bool ok = false;
    QPixmap pixmap;

    // 去掉前缀
    QStringList parts = base64Image.split(",");
    if (parts.length() == 2) {
        QString imageData = parts.at(1);
        QByteArray byteArray = QByteArray::fromBase64(imageData.toLocal8Bit());
        pixmap.loadFromData(byteArray);
    }
    if (pixmap.isNull()) {
       qDebug() << "pixmap is null";
       return ok;
    }

    // 获取当前时间
    QDateTime currentTime = QDateTime::currentDateTime();
    QString formattedTime = currentTime.toString("yyyyMMdd_hhmmss");
    // 使用格式化后的时间作为文件名
    QString fileName = "aiassistant_" + formattedTime + ".png";
    QString path = QStandardPaths::standardLocations(QStandardPaths::PicturesLocation).at(0);
    if (path.endsWith("/")) {
        path += fileName;
    } else {
        path += "/" +fileName;
    }

    QFileDialog *fileDialog = new QFileDialog(this, Qt::CustomizeWindowHint); // 设置窗口标题属性自定义
    fileDialog->setNameFilter(QLatin1String("PNG(*.png);;BMP(*.bmp);;FPEG(*.jpg)"));
    fileDialog->setDirectory(QStandardPaths::standardLocations(QStandardPaths::PicturesLocation).at(0));
    fileDialog->setAcceptMode(QFileDialog::AcceptSave);
    fileDialog->setLabelText(QFileDialog::Reject, tr("Cancel"));
    fileDialog->selectFile(path);

    while (!ok) {
        if (fileDialog->exec() == QFileDialog::Accepted) {
            QString savePath = fileDialog->selectedFiles().at(0);

            QFileInfo fileName = savePath;
            if (savePath.isNull()) {
                break;
            }

            if (fileName.suffix().isNull()) {
                savePath += QLatin1String(".png");
            }
            fileName = savePath;
            QString name = fileName.fileName();
            QString msg;
            QString message;
            if (!name.startsWith(QChar('.'), Qt::CaseInsensitive)) {
                ok = pixmap.save(savePath);
            }
            if (ok) {
                QString pathNoFile = savePath.left(savePath.lastIndexOf(QLatin1String("/")));
                message = QObject::tr("Image saved as ") + savePath;
                msg = "<a href=\"" + savePath + "\">" + message + "</a>";
            } else {
                if (name.contains(QChar('/'))) {
                    msg = QObject::tr("file name can not contains '/'");
                } else if (name.startsWith(QChar('.'), Qt::CaseInsensitive)) {
                    msg = QObject::tr("can not save file as hide file");
                } else if (name.length() > 255) {
                    msg = QObject::tr("can not save  because filename too long");
                } else {
                    message = QObject::tr("Error trying to save as ") + savePath;
                    msg = "<a href=\"" + savePath + "\">" + message + "</a>";
                }
                QMessageBox saveErrBox(
                    QMessageBox::Warning,
                    QObject::tr("Save Error"),
                    msg);
                saveErrBox.exec();
            }
        } else {
            return ok;
        }
    }
    return ok;
}

void CMsgPane::onSendDataCollect(const QString &eventCode, const QString &pageName)
{
       //埋点数据发送与处理
       // 初始化埋点数据，传入应用类型，桌面应用，时间类型，点击事件
       KTrackData *node = kdk_dia_data_init(KEVENTSOURCE_DESKTOP, KEVENT_CLICK);
       // 传入事件描述“对话框——发送消息”，所在界面“主界面”，上传事件
       //qDebug() << eventCode.toLocal8Bit().data() << pageName.toLocal8Bit().data();
       kdk_dia_upload_default(node,eventCode.toLocal8Bit().data(), pageName.toLocal8Bit().data());
       // 释放内存
       kdk_dia_data_free(node);

}

void CMsgPane::onSelectImage()
{
    QFileDialog m_fileDialog(this);
    m_fileDialog.update();
    QList<QUrl> list = m_fileDialog.sidebarUrls();
    int sidebarNum = 8; //最大添加U盘数，可以自己定义
    QString home = QDir::homePath().section("/", -1, -1);
    QString mnt = "/media/" + home + "/";
    QDir mntDir(mnt);
    mntDir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
    QFileInfoList filist = mntDir.entryInfoList();
    QList<QUrl> mntUrlList;
    for (int i = 0; i < sidebarNum && i < filist.size(); ++i) {
        QFileInfo fi = filist.at(i);
        //华为990、9a0需要屏蔽最小系统挂载的目录
        if (fi.fileName() == "2691-6AB8")
            continue;
        mntUrlList << QUrl("file://" + fi.filePath());
    }

    QFileSystemWatcher fsw(&m_fileDialog);
    fsw.addPath("/media/" + home + "/");
    connect(&fsw, &QFileSystemWatcher::directoryChanged, &m_fileDialog,
            [=, &sidebarNum, &mntUrlList, &list, &m_fileDialog](const QString path) {
                QDir wmntDir(path);
                wmntDir.setFilter(QDir::Dirs | QDir::NoDotAndDotDot);
                QFileInfoList wfilist = wmntDir.entryInfoList();
                mntUrlList.clear();
                for (int i = 0; i < sidebarNum && i < wfilist.size(); ++i) {
                    QFileInfo fi = wfilist.at(i);
                    //华为990、9a0需要屏蔽最小系统挂载的目录
                    if (fi.fileName() == "2691-6AB8")
                        continue;
                    mntUrlList << QUrl("file://" + fi.filePath());
                }
                m_fileDialog.setSidebarUrls(list + mntUrlList);
                m_fileDialog.update();
            });

    connect(&m_fileDialog, &QFileDialog::finished, &m_fileDialog, [=, &list, &m_fileDialog]() {
        m_fileDialog.setSidebarUrls(list);
    });

    //这行要添加，设置左侧导航
    m_fileDialog.setSidebarUrls(list + mntUrlList);
    //设置视图模式
    m_fileDialog.setViewMode(QFileDialog::Detail);
    //设置可以选择多个文件,默认为只能选择一个文件QFileDialog::ExistingFiles
    m_fileDialog.setFileMode(QFileDialog::ExistingFiles);
    m_fileDialog.setOption(QFileDialog::HideNameFilterDetails);

    QString file_path;
    QString format;
    QString openFile = tr("Open Image");
    QString fileType = tr("Image Files(");
    QVariant imagePath = QStandardPaths::writableLocation(QStandardPaths::PicturesLocation);
    QStringList supportFormats = {"tga", "svg", "gif", "apng", "ico", "xpm",
                                 "exr", "psd", "jfi", "jif", "j2k", "jp2", "jng", "wbmp", "xbm", "tiff", "tif", "webp",
                                 "pnm", "bmp", "dib", "sr",  "ras", "pbm", "pgm", "ppm",  "png", "jpg",  "jpe", "jpeg"};

    //构造打开条件
    format = "(";
    for (const QString &str : supportFormats)
        format += "*." + str + " ";
    format += ");;";

    const QString supportFormatsClassify = "(*.jpg *.jpe *.jpeg);;"
                                           "(*.pnm *.pbm);;"
                                           "(*.pgm *.ppm);;"
                                           "(*.png *.apng);;"
                                           "(*.ico);;(*.sr);;"
                                           "(*.ras);;(*.tga);;"
                                           "(*.svg);;(*.gif);;"
                                           "(*.webp);;(*.bmp *.dib);;"
                                           "(*.exr);;(*.psd);;(*.jfi *.jif);;"
                                           "(*.tif *.tiff);;"
                                           "(*.j2k *.jp2 *.jng);;(*.wbmp);;"
                                           "(*.xbm *.xpm)";
    //所有类型都放在一起，类型名显示不全，故增加分层，将同一类型的放在一起，分层显示
    format = format + supportFormatsClassify;

    //打开文件夹中的图片文件
    //m_fileDialog.move ((QApplication::desktop()->width() - m_fileDialog.width())/2,(QApplication::desktop()->height() - m_fileDialog.height())/2);
    file_path = m_fileDialog.getOpenFileName(nullptr, openFile, imagePath.toString(), format);

    //空校验，点击“取消”
    if (file_path.isEmpty()) {
        return;
    }
    onShowImage(file_path);
}

void CMsgPane::onShowImage(const QString &imagePath)
{

    QString base64Image = getBase64FromImagePath(imagePath);
    // qDebug() << "1111111111111111base64Image" <<base64Image;
    QString str = QString("data:image/png;base64," + base64Image);
    m_pViewChat->page()->runJavaScript(QString("sendImage('%1','%2')").arg(imagePath).arg(str));
}

void CMsgPane::onSendImage(const QString &imagePath)
{
    jsonObjToWeb["msgHead"] = msgHead;
    jsonObjToWeb["modelMsg"] = "ai";
    QJsonDocument aiJsonSendWebDoc(jsonObjToWeb);
    QString imageJsonMsgHeadToWeb = aiJsonSendWebDoc.toJson(QJsonDocument::Indented);
    pushMsg("addRecvMsg", imageJsonMsgHeadToWeb, "收到您的图片啦~需要我对这张图片做什么呢？");
    QString js_str = QString("setReplyOptions(true, '%1')").arg(imagePath);
    m_pViewChat->page()->runJavaScript(js_str);
    QString jsStr1 = QString("isReplyEnd(true)");
    m_pViewChat->page()->runJavaScript(jsStr1);
}

void CMsgPane::onEnhanceBlackAndWhite(const QString &imagePath)
{
//    QString blackAndWhiteImagePath = m_photoViewerPlugin->startScan(imagePath);
//    qDebug() << "startScan" << blackAndWhiteImagePath;
//    QString base64Image = getBase64FromImagePath(blackAndWhiteImagePath);
//    QString str = QString("data:image/png;base64," + base64Image);

    //AI助手生成
    jsonObjToWeb["msgHead"] = "";
    jsonObjToWeb["modelMsg"] = "ai";
    QJsonDocument aiJsonSendWebDoc(jsonObjToWeb);
    QString scheJsonMsgHeadToWeb = aiJsonSendWebDoc.toJson(QJsonDocument::Indented);
    scheJsonMsgHeadToWeb.replace('\n', "");
    scheJsonMsgHeadToWeb.replace("\\", "\\\\");
    scheJsonMsgHeadToWeb.replace('\n', "\\n");
    scheJsonMsgHeadToWeb.replace("\r", "\\r");
    scheJsonMsgHeadToWeb.replace("\"", "\\\"");
//    QString js_str = QString("recvImage('%1', '%2', '%3')").arg(scheJsonMsgHeadToWeb).arg(blackAndWhiteImagePath).arg(str);
    QString js_str = QString("recvImage('%1')").arg(scheJsonMsgHeadToWeb);
    m_pViewChat->page()->runJavaScript(js_str);
}

void CMsgPane::onCutOutImage(const QString &imagePath)
{
//    QString cutOutImagePath = m_photoViewerPlugin->startMatting(imagePath);
//    qDebug() << "startMatting" << cutOutImagePath;
//    QString base64Image = getBase64FromImagePath(cutOutImagePath);
//    QString str = QString("data:image/png;base64," + base64Image);

    //AI助手生成
    jsonObjToWeb["msgHead"] = "";
    jsonObjToWeb["modelMsg"] = "ai";
    QJsonDocument aiJsonSendWebDoc(jsonObjToWeb);
    QString scheJsonMsgHeadToWeb = aiJsonSendWebDoc.toJson(QJsonDocument::Indented);
    scheJsonMsgHeadToWeb.replace('\n', "");
    scheJsonMsgHeadToWeb.replace("\\", "\\\\");
    scheJsonMsgHeadToWeb.replace('\n', "\\n");
    scheJsonMsgHeadToWeb.replace("\r", "\\r");
    scheJsonMsgHeadToWeb.replace("\"", "\\\"");
//    QString js_str = QString("recvImage('%1', '%2', '%3')").arg(scheJsonMsgHeadToWeb).arg(cutOutImagePath).arg(str);
    QString js_str = QString("recvImage('%1')").arg(scheJsonMsgHeadToWeb);
    m_pViewChat->page()->runJavaScript(js_str);
}

QString CMsgPane::getBase64FromImagePath(const QString &imagePath) {
    QFile file(imagePath);
    if (!file.open(QIODevice::ReadOnly)) {
        // 错误处理：文件无法打开
        return QString();
    }

    QByteArray fileData = file.readAll();
    file.close();

    return fileData.toBase64();
}

void CMsgPane::onSendSpeechMsg(QString msg)
{
    pushMsg("addMsg", "", "");
    // 添加后端到后端语音对话识别标志
    QJsonObject speechJsonObj;
    speechJsonObj["speechFlagMsg"] = true;
    QJsonDocument aiJsonSendWebDoc(speechJsonObj);
    QString speechjsonMsg = aiJsonSendWebDoc.toJson(QJsonDocument::Indented);

    speech_send_message.append(msg);
    if (!speech_bubble_update_timer->isActive())
        speech_bubble_update_timer->start();

    m_chatOperator->soundFlag = true;
    this->onSendMsg(speechjsonMsg, msg);
}


void CMsgPane::initBubbleUpdateSpeechTimer()
{
    speech_bubble_update_timer = new QTimer;
    speech_bubble_update_timer->setInterval(40);
    connect(speech_bubble_update_timer, &QTimer::timeout, this, &CMsgPane::updateSpeechBubble);
}

/**
 * @brief CMsgPane::updateSpeechBubble 按字去更新内容
 */

void CMsgPane::updateSpeechBubble()
{
    bool loading = bubble_show_speech_count == speech_send_message.length();
    if (loading)
    {
        speech_send_message = "";
        bubble_show_speech_count = 0;
        speech_bubble_update_timer->stop();
        return;
    }
    QString show_msg = speech_send_message.left(loading ? bubble_show_speech_count : ++bubble_show_speech_count);
    if (!loading)
        loading_speech_dot_index = 0;
    QString dot_dot = "...";
    pushMsg("updateSpeechMsg", "", show_msg.append(dot_dot.left(loading ? loading_speech_dot_index++ / 10 % 4 : 0)));
}
