#ifndef CMSGPANE_H
#define CMSGPANE_H

#include <QWidget>
#include <QTimer>
#include <QWebEngineView>
#include <QTextEdit>
#include <QLabel>
#include <QJsonParseError>
#include <QJsonObject>
#include <QJsonArray>
#include <QDesktopWidget>
#include <QApplication>
#include <QMenuBar>
#include <QToolButton>
#include <QAction>
#include <QDebug>
#include <QMenu>
#include <QSystemTrayIcon>
#include <QPushButton>
#include <QDBusInterface>
#include <QDBusReply>
#include <QDBusObjectPath>
#include <QImage>
#include <QByteArray>
#include <QBuffer>
#include <QWebChannel>
#include <QDateTime>
#include <QDir>
#include <unistd.h>

#include "kasendbar.h"
#include "channelmessage.h"
#include "startuptools.h"
#include "ctrl.h"
#include "reader.h"
#include "recorder.h"
#include "runtimerecorder.h"
#include "meetingrecorder.h"
#include "kwebengineview.h"
#include "aitextgeneratedpic.h"

#include <kysdk/diagnosetest/libkydatacollect.h>

class ChatOperator;
class CPushButtonEx;
class Reader;
class Recorder;
class RuntimeRecorder;
class MeetingRecorder;
//class Matting;
//class ScannerPlugin;
//class PhotoViewerPlugin;

#define EMPTYMETTINGFLAG "EmptyMeetingFlag"

class CMsgPane : public QWidget
{
    Q_OBJECT
public:
    explicit CMsgPane(QWidget *parent = 0);

public slots:
    //返回主页
    void onBackBtnClicked();
    void onUrlChanged(const QUrl &url);
    //消息
    void onClearMsg();
    void onSendMsg(const QString &jsonMsgHead, const QString &msg);
    void onSoundMsg(const QString &msgHead, const QString &msg);
    void onRecvMsg(QString msg, bool is_end, int index, int errorCode);
    void onSendSpeechMsg(QString msg);
    void onSaveMessageList();
    void onLoadMessageList();
    //图像
    void onSelectImage();
    void onShowImage(const QString &imagePath);
    void onSendImage(const QString &imagePath);
    void onEnhanceBlackAndWhite(const QString &imagePath);
    void onCutOutImage(const QString &imagePath);
    //文生图
    void updateToShowAIPic(VisionImageData *imageData, QString errorMessage); // 更新AI文生图的图片
    bool onDownloadImage(const QString &base64Image);                         // 下载图片
    //埋点数据
    void onSendDataCollect(const QString &eventCode, const QString &pageName);


    //会话
    void saveDialog();
    void clearDialog();
    void onShowText(const QString text);
    void onShowSpeech(const QString text);
    void onAppendSpeech(const int speakerID, const QString text);
    void OnWebViewLoaded();
    void userAccountsChanged();
    void loadPlugin(QString pluginHtmlFile);
    //添加到日历
    void onAddToSchedule(QString msg, bool is_end, int index);
    //发送整段语音
    void onSendAudio();
    void onStopAudio();
    void onCancelAudio();
    //发送实时语音
    void onSendRuntimeAudio(QString lan, QString target);
    void onStopRuntimeAudio();
    //语音朗读
    void onStartRead(const QString &msg, int index);
    void onPauseRead(int index);
    void onContinueRead(int index);
    void onReadEnd(int index);
    //语音听写
    void onClearDictation();
    void onCopyDictation();
    //发送会议录音
    void onSendMeetingAudio();
    void onPauseMeetingAudio();
    void onContinueMeetingAudio();
    void onStopMeetingAudio();
    //发言人重命名
    void onRename(QString originalName);
    //会议列表
    void onMeetingList(QString str);
    //处理会议原文
    void processMessageList(QString jsonStr);
    void onGenerateSummary(QString msg, bool is_end, int index);

    void onStopReply();

private:
    void createAllChildWnd();
    void initCtrl();
    void initSolts();
    void relayout();
    void sendGetRequest(QString strMsg);
    void initChatOperator();
    void initBubbleUpdateTimer();
    void initBubbleUpdateSpeechTimer();
    void initCurrentUserName();
    void initCurrentUserIcon();
    void pushMsg(QString func, QString jsonMsgHeadToWeb, QString msg);
    void saveDialogToFile(const QString &dialogJsonString);
    void saveMessageToFile(const QString &msgJsonString);
    void saveMeetingToFile();    
    //保存会议到临时文件
    void saveMeetingToTempFile();

    QString getBase64FromImagePath(const QString &imagePath); // 从图片路径获取到Base64编码

signals:
    void sigSetup();
    void sigSaveDialog(QString filePath);
    void sigUrlChanged(const QUrl &url);

private slots:
    void updateBubble();
    void updateSpeechBubble();

private:
    QWebChannel *m_channel = nullptr;
    ChannelMessage *m_channelMsg = nullptr;
//    QWebEngineView *m_pViewChat = nullptr;
    KWebEngineView *m_pViewChat = nullptr;
    //CTextEditEx *m_textEdit;
    Reader *m_reader = nullptr;
    Recorder *m_recorder = nullptr;
    RuntimeRecorder *m_runtimeRecorder = nullptr;
    MeetingRecorder *m_meetingRecorder = nullptr;
    AITextGeneratedPic *m_AIDrawing = nullptr;
//    Matting *m_aiMatting = nullptr;
//    ScannerPlugin *m_aiScannerPlugin = nullptr;
//    PhotoViewerPlugin *m_photoViewerPlugin = nullptr;

    CPushButtonEx *m_btnMore = nullptr;
    CPushButtonEx *m_btnVoice = nullptr;
    QLabel *m_labSeparatorLine = nullptr;
    QAction *confAction = nullptr;
    //NLP语言模型
    ChatOperator *m_chatOperator = nullptr;
    //文字发送的消息
    QString send_message;
    //语音发送消息到前端
    QString speech_send_message;
    int bubble_show_speech_count;
    uint loading_speech_dot_index;
    QTimer *speech_bubble_update_timer = nullptr;
    //返回消息
    bool new_reply = true;
    QString recv_message;
    int bubble_show_count;
    uint loading_dot_index;
    QTimer *bubble_update_timer = nullptr;
    //会议
    QString m_summary;
    QString m_origin;
    QString m_finishedSpeech;
    QString m_tempMettingFileName;
    //消息头
    QString msgHead;
    bool updateReplyFlag;
    bool speechFlag;
    QJsonObject jsonObjToQt;
    QJsonObject jsonObjToWeb;
};

#endif // CMSGPANE_H
