#ifndef READER_H
#define READER_H

#include <QObject>
#include <QFile>
#include <QMediaPlayer>
#include <QMutex>
#include <kylin-ai/ai-base/speech.h>
#include "msgpane.h"
#include "channelmessage.h"

#include <QAudioOutput>
#include <QAudioFormat>
#include <QIODevice>
#include <QThread>
#include <QTemporaryFile>
#include <QQueue>
#include <QMutex>
#include <QWaitCondition>
#include <QMediaPlayer>
#include <QQueue>
#include <QByteArray>

class AudioPlayer : public QObject
{
    Q_OBJECT

public:
    AudioPlayer(QObject *parent = nullptr)
        : QObject(parent), m_mediaPlayer(new QMediaPlayer(this))
    {
        connect(m_mediaPlayer, &QMediaPlayer::mediaStatusChanged, this, &AudioPlayer::onMediaStatusChanged);
    }

    ~AudioPlayer()
    {
        // 停止媒体播放器并清理资源
        m_mediaPlayer->stop();
        delete m_mediaPlayer;
    }

    void enqueueAudio(const QByteArray &audioData)
    {
        m_playQueue.enqueue(audioData);
        if (m_mediaPlayer->state() != QMediaPlayer::PlayingState) {
            playNextAudio();
        }
    }

signals:
    void audioPlaybackFinished();

private slots:
    void onMediaStatusChanged(QMediaPlayer::MediaStatus status)
    {
        //qDebug()<<"status ========= "<<status <<"EndOfMedia = "<<QMediaPlayer::EndOMedia;
        if (status == QMediaPlayer::EndOfMedia) {
            // 播放结束，准备下一次播放
            playNextAudio();
        }
        else if (status == QMediaPlayer::InvalidMedia)
        {
            qDebug()<<"数据异常";
            //m_mediaPlayer->play();
        }
    }

    void playNextAudio()
    {
        if (!m_playQueue.isEmpty()) {
            QByteArray audioData = m_playQueue.dequeue();
            // 假设这里有一个将QByteArray转换为QMediaPlayer可以播放的格式的方法
            // 例如，将数据写入一个临时文件并设置给媒体播放器
            QTemporaryFile tempFile;
            if (tempFile.open()) {
                tempFile.write(audioData);
                tempFile.close();
                m_mediaPlayer->setMedia(QUrl::fromLocalFile(tempFile.fileName()));

                qDebug()<<"QUrl::fromLocalFile(tempFile.fileName()) = "<<QUrl::fromLocalFile(tempFile.fileName());
                m_mediaPlayer->play();
                qDebug()<<"m_textSoundFlag = "<<m_textSoundFlag;
            } else {
                qDebug() << "Failed to open temporary file";
            }
        } else {
            // 队列为空，可以发出播放结束信号或进行其他处理
            qDebug()<<"队列为空，可以发出播放结束信号或进行其他处理 m_textSoundFlag= "<<m_textSoundFlag;
            emit audioPlaybackFinished();
        }
    }
public:
    bool m_textSoundFlag = false;

private:
    QMediaPlayer *m_mediaPlayer;
    QQueue<QByteArray> m_playQueue; // 使用QQueue作为音频数据队列
};


class Reader : public QObject
{
    Q_OBJECT
public:
    explicit Reader(QObject *parent = nullptr);
    ~Reader();
    void startReading();
    void pauseReading(int index);
    void continueReading(int index);
    void sendText(const QString &msg, int index);
    void audioAppend(QByteArray tmp);
    void soundApplay(const char *data, int data_size);
    void reinitSession();
    void closeSession();

    QFile *m_file;
    bool m_soundFlag = false;
    bool m_textSoundFlag = true;
    AudioPlayer *audio;

signals:
    void sigReadEnd(int index);

public slots:
    void onPlayerStateChanged(QMediaPlayer::State state);
    void textToSoundOutput(const QString &message, bool is_end, int index);

private:
    QMediaPlayer *m_player = nullptr;
    void *session = nullptr;
    SpeechSession soundSession = nullptr;
    int m_index = -1;

    bool initSession();
    void initConnect();
    QString generateUniqueFileName();
};

#endif // READER_H
