#include "kainstructionpage.h"
#include "templatebutton.h"
#include "global/kaglobal.h"
#include "libukcc/widgets/SettingWidget/settinggroup.h"

#include <QLabel>
#include <QSharedPointer>
#include <QGridLayout>

class KAInstructionPage::Private
{
public:
    QLabel *mInstructTitleLabel = nullptr;
    UkccFrame *mTemplateFrame = nullptr;
    QLabel *mTemplateLabel = nullptr;
    int templateNumber = 10;
};

KAInstructionPage::KAInstructionPage(QWidget *parent)
    : QWidget{parent},
      d_ptr(make_shared<Private>())
{
    initLayout();
}

KAInstructionPage::~KAInstructionPage()
{

}


void KAInstructionPage::initLayout()
{
    QVBoxLayout *base_vlayout = new QVBoxLayout(this);
    base_vlayout->setContentsMargins(12, 10, 16, 0);

    QString templateNumberStr = QString::number(d_ptr->templateNumber);
    QString titleTextStr = tr("Instruction Template") + " (" + templateNumberStr + ")";
    d_ptr->mInstructTitleLabel = new QLabel(this);
    d_ptr->mInstructTitleLabel->setText(titleTextStr);
    d_ptr->mInstructTitleLabel->setContentsMargins(20, 0, 0, 0);
    QFont font("Noto Sans CJK SC", 11);
    d_ptr->mInstructTitleLabel->setFont(font);
    base_vlayout->addWidget(d_ptr->mInstructTitleLabel);

    d_ptr->mTemplateFrame = new UkccFrame(this);
    d_ptr->mTemplateFrame->setMinimumSize(QSize(496, 96));
    d_ptr->mTemplateFrame->setMaximumSize(QSize(16777215, 96));
    d_ptr->mTemplateFrame->setFrameShape(QFrame::Box);
    d_ptr->mTemplateFrame->setObjectName(QString("templateFrame"));

    TemplateButton *template1 = new TemplateButton(tr("文本扩写"), this);
    TemplateButton *template2 = new TemplateButton(tr("文本润色"), this);
    TemplateButton *template3 = new TemplateButton(tr("中英文翻译"), this);
    TemplateButton *template4 = new TemplateButton(tr("内容质检"), this);
    TemplateButton *template5 = new TemplateButton(tr("总结概括"), this);
    TemplateButton *template6 = new TemplateButton(tr("邮件回复"), this);
    TemplateButton *template7 = new TemplateButton(tr("工作总结"), this);
    TemplateButton *template8 = new TemplateButton(tr("代码生成"), this);
    TemplateButton *template9 = new TemplateButton(tr("新建日程"), this);
    TemplateButton *template10 = new TemplateButton(tr("英中文翻译"), this);

    QGridLayout *tableLayout = new QGridLayout;
    tableLayout->setHorizontalSpacing(8);
    tableLayout->addWidget(template1, 0, 0, Qt::AlignCenter);
    tableLayout->addWidget(template2, 0, 1, Qt::AlignCenter);
    tableLayout->addWidget(template3, 0, 2, Qt::AlignCenter);
    tableLayout->addWidget(template4, 0, 3, Qt::AlignCenter);
    tableLayout->addWidget(template5, 0, 4, Qt::AlignCenter);

    tableLayout->addWidget(template6, 1, 0, Qt::AlignCenter);
    tableLayout->addWidget(template7, 1, 1, Qt::AlignCenter);
    tableLayout->addWidget(template8, 1, 2, Qt::AlignCenter);
    tableLayout->addWidget(template9, 1, 3, Qt::AlignCenter);
    tableLayout->addWidget(template10, 1, 4, Qt::AlignCenter);

    d_ptr->mTemplateFrame->setLayout(tableLayout);
    base_vlayout->addWidget(d_ptr->mInstructTitleLabel);
    base_vlayout->setSpacing(8);
    base_vlayout->addWidget(d_ptr->mTemplateFrame);
    base_vlayout->addStretch();
}

void KAInstructionPage::initConnection()
{

}

