#include "kamodelitem.h"

#include <QMenu>
#include <QLabel>
#include <QVariant>
#include <QToolButton>
#include <QHBoxLayout>

class KAModelItem::Private
{
public:
    QLabel *nameLabel = nullptr;        // 模型名称
    QToolButton *moreButton = nullptr;  // 更多
    QMenu *menu = nullptr;              // 菜单
    QLabel *selectedIconLabel = nullptr;// 已选择状态
    ModelTypeId modelTypeId;
    ModelVendorId modelVendorId;

};

KAModelItem::KAModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId, QWidget *parent)
    : QWidget{parent},
      d_ptr(make_shared<Private>())
{
    d_ptr->modelTypeId = modelTypeId;
    d_ptr->modelVendorId = modelVendorId;
    initLayout();
    initMenu();
}

KAModelItem::~KAModelItem()
{

}

void KAModelItem::initLayout()
{
    d_ptr->nameLabel = new QLabel;
    d_ptr->nameLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
    QString model_name = "";

    if (d_ptr->modelTypeId == NLP_TYPE) {
        switch (d_ptr->modelVendorId) {
            case BAIDU_TYPE:
                model_name = tr("BAIDU-ERNIE 4.0");                    // 百度-ERNIE 4.0
                break;
            case XUNFEI_TYPE:
                model_name = tr("XUNFEI-Starfire Model V3.5");         // 迅飞-星火大模型 V3.5
                break;
            default:
                model_name = tr("No Vendor");
                break;
        }
    } else if (d_ptr->modelTypeId == VISION_TYPE) {
        switch (d_ptr->modelVendorId) {
            case BAIDU_TYPE:
                model_name = tr("BAIDU-AI Painting-Advanced Edition"); // 百度-AI作画-高级版
                break;
            case XUNFEI_TYPE:
                model_name = tr("XUNFEI--Image Generation");           // 迅飞--图片生成
                break;
            default:
                model_name = tr("No Vendor");
                break;
        }
    } else if (d_ptr->modelTypeId == SPEECH_TYPE) {
        switch (d_ptr->modelVendorId) {
            case BAIDU_TYPE:
                model_name = tr("BAIDU-Voice Big Model");              // 百度-语音大模型
                break;
            case XUNFEI_TYPE:
                model_name = tr("XUNFEI-Voice Big Model");             // 迅飞-语音大模型
                break;
            default:
                model_name = tr("No Vendor");
                break;
        }
    }
    d_ptr->nameLabel->setText(model_name);

    d_ptr->moreButton = new QToolButton;
    d_ptr->moreButton->setFixedSize(38, 38);
    d_ptr->moreButton->setProperty("useButtonPalette", true);
    d_ptr->moreButton->setAutoRaise(true);
    d_ptr->moreButton->setPopupMode(QToolButton::InstantPopup);
    d_ptr->moreButton->setIcon(QIcon::fromTheme("view-more-horizontal-symbolic"));
    d_ptr->moreButton->setFocusPolicy(Qt::NoFocus);

    d_ptr->selectedIconLabel = new QLabel();
//    d_ptr->selectedIconLabel->setPixmap(QPixmap(":/icon/icon/selected.png"));
//    d_ptr->selectedIconLabel->setFixedSize(16, 16);
    QIcon selectIcon = QIcon::fromTheme("ukui-selected");
    d_ptr->selectedIconLabel->setPixmap(selectIcon.pixmap(selectIcon.actualSize(QSize(16, 16))));
    const QByteArray id(THEME_QT_SCHEMA);
    QGSettings *mQtSettings = new QGSettings(id, QByteArray(), this);
    connect(mQtSettings, &QGSettings::changed, this, [=](QString key) {
        if (key == "iconThemeName")
            d_ptr->selectedIconLabel->setPixmap(selectIcon.pixmap(selectIcon.actualSize(QSize(16, 16))));
    });
    QFrame *frame = new QFrame(this);
    frame->setFixedSize(16,16);
    QHBoxLayout *frameLayout = new QHBoxLayout;
    frameLayout->setMargin(0);
    frame->setLayout(frameLayout);
    frameLayout->addWidget(d_ptr->selectedIconLabel);
    d_ptr->selectedIconLabel->setVisible(false);

    QHBoxLayout *hLayout = new QHBoxLayout;
    hLayout->setContentsMargins(16, 10, 16, 10);
    hLayout->addWidget(d_ptr->nameLabel);
    hLayout->addStretch();
    hLayout->addWidget(d_ptr->moreButton);
    hLayout->addWidget(frame);
    this->setLayout(hLayout);
}

void KAModelItem::initMenu()
{
    d_ptr->menu = new QMenu();
    d_ptr->moreButton->setMenu(d_ptr->menu);

    QAction *actionEdit = new QAction(QIcon::fromTheme("document-edit-symbolic"), tr("Edit"));
    actionEdit->setDisabled(true);
//    connect(actionEdit, &QAction::triggered, [this]{emit sigEdit(d_ptr->modelTypeId, d_ptr->modelVendorId);});

    QAction *actionDelete = new QAction(QIcon::fromTheme("edit-delete-symbolic"), tr("Delete"));
    connect(actionDelete, &QAction::triggered, [this]{emit sigDelete(d_ptr->modelTypeId, d_ptr->modelVendorId);});

    d_ptr->menu->addAction(actionEdit);
    d_ptr->menu->addAction(actionDelete);
}

void KAModelItem::showSelectedIcon(bool flag)
{
    d_ptr->selectedIconLabel->setVisible(flag);

}

bool KAModelItem::isSelected()
{
    return d_ptr->selectedIconLabel->isVisible();
}

ModelTypeId KAModelItem::getModelTypeId()
{
    return d_ptr->modelTypeId;
}

ModelVendorId KAModelItem::getModelVendorId()
{
    return d_ptr->modelVendorId;
}
