#include "kamultimodelsettingpage.h"

#include <QLabel>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGridLayout>
#include <QSharedPointer>
#include <QPushButton>
#include <kswitchbutton.h>
#include <QIcon>
#include <QComboBox>
#include <QScrollArea>
#include <QScrollBar>

#include "global/kaglobal.h"
#include "libukcc/widgets/SettingWidget/addbutton.h"
#include "libukcc/widgets/SettingWidget/settinggroup.h"

#include <kylin-ai/config.h>

const QString CONFIGPATH = "/.config/kylin-aiassistant/modelsetting.json";

class KAMultiModelSettingPage::Private
{
public:
    QScrollArea *m_scrollArea = nullptr;       // 滚动条
    QWidget *m_scrollContent = nullptr;
    QVBoxLayout *m_contentLayout = nullptr;

    QLabel *labelDeployMethod = nullptr;       // 部署方式
    ModelCardWidget *m_cloudModel = nullptr;   // 云端模型
    ModelCardWidget *m_privateModel = nullptr; // 私有化模型
    ModelCardWidget *m_localModel = nullptr;   // 本地模型
    ModelCardWidget *m_customModel = nullptr;  // 自定义
    ModelCardWidget *m_cloudModelHighlight = nullptr;   // 云端模型选中状态
    ModelCardWidget *m_privateModelHighlight = nullptr; // 私有化模型选中状态
    ModelCardWidget *m_localModelHighlight = nullptr;   // 本地模型选中状态
    ModelCardWidget *m_customModelHighlight = nullptr;  // 自定义选中状态

    QWidget *m_modelWidget = nullptr;
    QVBoxLayout *m_baseLayout = nullptr;
    QHBoxLayout *m_modelLayout = nullptr;

    QWidget *m_BigModelWidget = nullptr;          //大模型配置界面
    QVBoxLayout *m_BigModelLayout = nullptr;

    QWidget *m_nlpModelWidget = nullptr;          //NLP语言大模型
    QHBoxLayout *m_nlpHModelLayout = nullptr;
    QVBoxLayout *m_nlpVModelLayout = nullptr;
    QLabel * nlp_LanguageModel = nullptr;

    QWidget *m_visionModelWidget = nullptr;        //vision视觉大模型
    QHBoxLayout *m_visionHModelLayout = nullptr;
    QVBoxLayout *m_visionVModelLayout = nullptr;
    QLabel * vision_VisualModel = nullptr;

    QWidget *m_speechModelWidget = nullptr;       //SPEECH语音大模型
    QHBoxLayout *m_speechHModelLayout = nullptr;
    QVBoxLayout *m_speechVModelLayout = nullptr;
    QLabel * speech_VoiceModel = nullptr;

    QPushButton *nlpAddButton = nullptr;
    QPushButton *visionAddButton = nullptr;
    QPushButton *speechAddButton = nullptr;

    QWidget *m_privateModelWidget = nullptr;  //私有化模型界面

    QMap<ModelVendorId, S_Model> m_nlpModelMap;
    QMap<ModelVendorId, S_Model> m_visionModelMap;
    QMap<ModelVendorId, S_Model> m_speechModelMap;

    QMap<ModelVendorId, KAModelItem*> m_nlpModelItemMap;
    QMap<ModelVendorId, KAModelItem*> m_visionModelItemMap;
    QMap<ModelVendorId, KAModelItem*> m_speechModelItemMap;

    QMap<ModelVendorId, UkccFrame*> m_nlpModelMapUI;
    QMap<ModelVendorId, UkccFrame*> m_visionModelMapUI;
    QMap<ModelVendorId, UkccFrame*> m_speechModelMapUI;

    SettingGroup *m_nlpModelGroup = nullptr;
    SettingGroup *m_visionModelGroup = nullptr;
    SettingGroup *m_speechModelGroup = nullptr;

    int nlpSelectedModel = -1;
    int visionSelectedModel = -1;
    int speechSelectedModel = -1;
    int continuousSelected = -1;
    int realtimeSelected = -1;
};

KAMultiModelSettingPage::KAMultiModelSettingPage(QWidget *parent)
    : QWidget{parent},
      d_ptr(make_shared<Private>())
{

    initLayout();
    loadConfigFromJson();
    reinitLayout();
}

KAMultiModelSettingPage::~KAMultiModelSettingPage()
{

}

void KAMultiModelSettingPage::initLayout()
{
    this->setFixedSize(544, 400);
    d_ptr->m_scrollArea = new QScrollArea(this);
    d_ptr->m_scrollArea->setWidgetResizable(true);
    d_ptr->m_scrollContent = new QWidget(this);
    d_ptr->m_scrollContent->setFixedWidth(544);
    d_ptr->m_scrollContent->setMinimumHeight(600);
    d_ptr->m_scrollContent->setContentsMargins(0, 0, 0, 0);
    d_ptr->m_scrollArea->setWidget(d_ptr->m_scrollContent);

    d_ptr->m_scrollArea->setFrameStyle(QFrame::NoFrame); // 无边框
    d_ptr->m_scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded); // 根据需要显示
    d_ptr->m_scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff); // 不显示水平滚动条
    d_ptr->m_scrollArea->verticalScrollBar()->setContextMenuPolicy(Qt::NoContextMenu);
    d_ptr->m_scrollArea->verticalScrollBar()->setProperty("drawScrollBarGroove", false);
    d_ptr->m_scrollArea->verticalScrollBar()->setVisible(false);

    d_ptr->m_contentLayout = new QVBoxLayout(d_ptr->m_scrollContent);
    d_ptr->m_contentLayout->setContentsMargins(0, 0, 0, 0);

    d_ptr->m_baseLayout = new QVBoxLayout;
    d_ptr->m_baseLayout->setAlignment(Qt::AlignTop);
    d_ptr->m_baseLayout->addWidget(d_ptr->m_scrollArea);

    d_ptr->labelDeployMethod = new QLabel;
    d_ptr->labelDeployMethod->setText(tr("Deployment method"));
    d_ptr->labelDeployMethod->setContentsMargins(22, 0, 10, 0);
    QFont font("Noto Sans CJK SC", 11);
    d_ptr->labelDeployMethod->setFont(font);
    d_ptr->m_contentLayout->addWidget(d_ptr->labelDeployMethod);

    d_ptr->m_cloudModel = new ModelCardWidget(tr("Cloud Model"), QPixmap(":/icon/icon/setting/cloudModel.svg"), true);
    d_ptr->m_privateModel = new ModelCardWidget(tr("Private Model"), QPixmap(":/icon/icon/setting/privateModel.svg"), true);
    d_ptr->m_localModel = new ModelCardWidget(tr("Local Model"), QPixmap(":/icon/icon/setting/localModel.svg"), true);
    // d_ptr->m_customModel = new ModelCardWidget(tr("Custom"), QPixmap(":/icon/icon/setting/custom.svg"), true);

    d_ptr->m_cloudModelHighlight = new ModelCardWidget(tr("Cloud Model"), QPixmap(":/icon/icon/setting/cloudModel-highlight.svg"), false);
    d_ptr->m_privateModelHighlight = new ModelCardWidget(tr("Private Model"), QPixmap(":/icon/icon/setting/privateModel-highlight.svg"), false);
    d_ptr->m_localModelHighlight = new ModelCardWidget(tr("Local Model"), QPixmap(":/icon/icon/setting/localModel-highlight.svg"), false);
    // d_ptr->m_customModelHighlight = new ModelCardWidget(tr("Custom"), QPixmap(":/icon/icon/setting/custom-highlight.svg"), false);

    d_ptr->m_cloudModel->installEventFilter(this);
    d_ptr->m_privateModel->installEventFilter(this);
    d_ptr->m_localModel->installEventFilter(this);
    // d_ptr->m_customModel->installEventFilter(this);

    d_ptr->m_cloudModel->installEventFilter(this);
    d_ptr->m_privateModel->installEventFilter(this);
    d_ptr->m_localModel->installEventFilter(this);
    // d_ptr->m_customModel->installEventFilter(this);

    d_ptr->m_modelWidget = new QWidget;
    d_ptr->m_modelLayout = new QHBoxLayout;

    d_ptr->m_modelWidget = new QWidget;
    d_ptr->m_modelWidget->setFixedHeight(68);
    d_ptr->m_modelLayout = new QHBoxLayout;
    d_ptr->m_modelLayout->addWidget(d_ptr->m_cloudModel);
    d_ptr->m_modelLayout->addWidget(d_ptr->m_cloudModelHighlight);
    d_ptr->m_modelLayout->addSpacing(1);
    d_ptr->m_modelLayout->addWidget(d_ptr->m_privateModel);
    d_ptr->m_modelLayout->addWidget(d_ptr->m_privateModelHighlight);
    d_ptr->m_modelLayout->addSpacing(1);
    d_ptr->m_modelLayout->addWidget(d_ptr->m_localModel);
    d_ptr->m_modelLayout->addWidget(d_ptr->m_localModelHighlight);
    // d_ptr->m_modelLayout->addWidget(d_ptr->m_customModel);
    // d_ptr->m_modelLayout->addWidget(d_ptr->m_customModelHighlight);
    d_ptr->m_modelLayout->setContentsMargins(6, 0, 20, 0);
    d_ptr->m_modelWidget->setLayout(d_ptr->m_modelLayout);
    d_ptr->m_contentLayout->addWidget(d_ptr->m_modelWidget);

    d_ptr->m_cloudModel->setVisible(true);
    d_ptr->m_privateModel->setVisible(true);
    d_ptr->m_localModel->setVisible(true);
    // d_ptr->m_customModel->setVisible(true);
    d_ptr->m_cloudModelHighlight->setVisible(false);
    d_ptr->m_privateModelHighlight->setVisible(false);
    d_ptr->m_localModelHighlight->setVisible(false);
    // d_ptr->m_customModelHighlight->setVisible(false);

    initConfigModelPage();
    initPrivateModelPage();
    d_ptr->m_contentLayout->addStretch();
    d_ptr->m_baseLayout->addStretch();
    this->setLayout(d_ptr->m_baseLayout);
}

void KAMultiModelSettingPage::initConfigModelPage()
{
    // 配置大模型界面
    d_ptr->m_BigModelWidget = new QWidget;
    d_ptr->m_BigModelWidget->setFixedWidth(544);
    d_ptr->m_BigModelWidget->setMinimumHeight(300);
    d_ptr->m_BigModelLayout = new QVBoxLayout;

    // 创建NLP语言大模型
    d_ptr->m_nlpModelWidget = new QWidget;
    d_ptr->m_nlpVModelLayout = new QVBoxLayout;
    d_ptr->m_nlpVModelLayout->setContentsMargins(0, 0, 0, 0);
    d_ptr->m_nlpHModelLayout = new QHBoxLayout;
    d_ptr->m_nlpHModelLayout->setContentsMargins(16, 4, 48, 4);
    d_ptr->nlp_LanguageModel = new QLabel;
    d_ptr->nlp_LanguageModel->setText(tr("NLP Language Model"));
    QFont font("Noto Sans CJK SC", 11);
    d_ptr->nlp_LanguageModel->setFont(font);
    d_ptr->m_nlpHModelLayout->addWidget(d_ptr->nlp_LanguageModel);
    d_ptr->m_nlpHModelLayout->addStretch();
    // 创建添加按钮
    d_ptr->nlpAddButton = new QPushButton;
    d_ptr->nlpAddButton->setFixedSize(80, 28);
    QIcon addIcon(":/icon/icon/setting/modelAdd.svg");
    d_ptr->nlpAddButton->setIcon(addIcon);
    d_ptr->nlpAddButton->setIconSize(QSize(16, 16));
    d_ptr->nlpAddButton->setText(tr("Add"));
    d_ptr->m_nlpHModelLayout->addWidget(d_ptr->nlpAddButton);

    d_ptr->m_nlpVModelLayout->addLayout(d_ptr->m_nlpHModelLayout);

    d_ptr->m_nlpModelGroup = new SettingGroup;
    {
        d_ptr->m_nlpVModelLayout->addWidget(d_ptr->m_nlpModelGroup);

    }
    d_ptr->m_nlpModelWidget->setLayout(d_ptr->m_nlpVModelLayout);
    d_ptr->m_BigModelLayout->addWidget(d_ptr->m_nlpModelWidget);


    // 创建vision视觉大模型
    d_ptr->m_visionModelWidget = new QWidget;
    d_ptr->m_visionVModelLayout = new QVBoxLayout;
    d_ptr->m_visionVModelLayout->setContentsMargins(0, 0, 0, 0);
    d_ptr->m_visionHModelLayout = new QHBoxLayout;
    d_ptr->m_visionHModelLayout->setContentsMargins(16, 4, 48, 4);
    d_ptr->vision_VisualModel = new QLabel;
    d_ptr->vision_VisualModel->setText(tr("VISION Visual Model"));
    d_ptr->vision_VisualModel->setFont(font);
    d_ptr->m_visionHModelLayout->addWidget(d_ptr->vision_VisualModel);
    d_ptr->visionAddButton = new QPushButton;
    d_ptr->visionAddButton->setFixedSize(80, 28);
    d_ptr->visionAddButton->setIcon(addIcon);
    d_ptr->visionAddButton->setIconSize(QSize(16, 16));
    d_ptr->visionAddButton->setText(tr("Add"));
    d_ptr->m_visionHModelLayout->addWidget(d_ptr->visionAddButton);

    d_ptr->m_visionVModelLayout->addLayout(d_ptr->m_visionHModelLayout);

    d_ptr->m_visionModelGroup = new SettingGroup;
    {
        d_ptr->m_visionVModelLayout->addWidget(d_ptr->m_visionModelGroup);

    }
    d_ptr->m_visionModelWidget->setLayout(d_ptr->m_visionVModelLayout);
    d_ptr->m_BigModelLayout->addWidget(d_ptr->m_visionModelWidget);

    // 创建SPEECH语音大模型
    d_ptr->m_speechModelWidget = new QWidget;
    d_ptr->m_speechVModelLayout = new QVBoxLayout;
    d_ptr->m_speechVModelLayout->setContentsMargins(0, 0, 0, 0);
    d_ptr->m_speechHModelLayout = new QHBoxLayout;
    d_ptr->m_speechHModelLayout->setContentsMargins(16, 4, 48, 4);
    d_ptr->speech_VoiceModel = new QLabel;
    d_ptr->speech_VoiceModel->setText(tr("SPEECH Model"));
    d_ptr->speech_VoiceModel->setFont(font);
    d_ptr->m_speechHModelLayout->addWidget(d_ptr->speech_VoiceModel);
    d_ptr->speechAddButton = new QPushButton;
    d_ptr->speechAddButton->setFixedSize(80, 28);
    d_ptr->speechAddButton->setIcon(addIcon);
    d_ptr->speechAddButton->setIconSize(QSize(16, 16));
    d_ptr->speechAddButton->setText(tr("Add"));
    d_ptr->m_speechHModelLayout->addWidget(d_ptr->speechAddButton);

    d_ptr->m_speechVModelLayout->addLayout(d_ptr->m_speechHModelLayout);

    d_ptr->m_speechModelGroup = new SettingGroup;
    {
        d_ptr->m_speechVModelLayout->addWidget(d_ptr->m_speechModelGroup);

    }
    d_ptr->m_speechModelWidget->setLayout(d_ptr->m_speechVModelLayout);
    d_ptr->m_BigModelLayout->addWidget(d_ptr->m_speechModelWidget);

    connect(d_ptr->nlpAddButton, &QPushButton::clicked, this, [this](){
        this->nlpAddModel();
    });
    connect(d_ptr->visionAddButton, &QPushButton::clicked, this, [this](){
        this->visionAddModel();
    });
    connect(d_ptr->speechAddButton, &QPushButton::clicked, this, [this](){
        this->speechAddModel();
    });

    d_ptr->m_nlpModelWidget->setVisible(false);
    d_ptr->m_visionModelWidget->setVisible(false);
    d_ptr->m_speechModelWidget->setVisible(false);

    d_ptr->m_modelLayout->addStretch();
    d_ptr->m_BigModelLayout->addStretch();

    d_ptr->m_BigModelWidget->setLayout(d_ptr->m_BigModelLayout);
    d_ptr->m_contentLayout->addWidget(d_ptr->m_BigModelWidget);
}


QJsonObject KAMultiModelSettingPage::mapToJson(const QMap<ModelVendorId, S_Model>& map) {
    QJsonObject obj;
    for (auto it = map.constBegin(); it != map.constEnd(); ++it) {
        QJsonObject modelObj;
        modelObj["modelTypeId"] = it.value().modelTypeId;
        modelObj["modelVendorId"] = it.value().modelVendorId;
        modelObj["appId"] = it.value().appId;
        modelObj["apiKey"] = it.value().apiKey;
        modelObj["apiSecret"] = it.value().apiSecret;

        modelObj["appIdContinuous"] = it.value().appIdContinuous;
        modelObj["apiKeyContinuous"] = it.value().apiKeyContinuous;
        modelObj["secretKeyContinuous"] = it.value().secretKeyContinuous;

        modelObj["appIdRealTime"] = it.value().appIdRealTime;
        modelObj["apiKeyRealTime"] = it.value().apiKeyRealTime;

        obj[QString::number(it.key())] = modelObj;
    }
    return obj;
}

QMap<ModelVendorId, S_Model> KAMultiModelSettingPage::jsonToMap(const QJsonObject& obj) {
    QMap<ModelVendorId, S_Model> map;
    for (auto it = obj.constBegin(); it != obj.constEnd(); ++it) {
        ModelVendorId key = static_cast<ModelVendorId>(it.key().toInt());
        QJsonObject modelObj = it.value().toObject();
        S_Model model;
        model.modelTypeId = static_cast<ModelTypeId>(modelObj["modelTypeId"].toInt());
        model.modelVendorId = static_cast<ModelVendorId>(modelObj["modelVendorId"].toInt());

        model.appId = modelObj["appId"].toString();
        model.apiKey = modelObj["apiKey"].toString();
        model.apiSecret = modelObj["apiSecret"].toString();

        model.appIdContinuous = modelObj["appIdContinuous"].toString();
        model.apiKeyContinuous = modelObj["apiKeyContinuous"].toString();
        model.secretKeyContinuous = modelObj["secretKeyContinuous"].toString();

        model.appIdRealTime = modelObj["appIdRealTime"].toString();
        model.apiKeyRealTime = modelObj["apiKeyRealTime"].toString();

        map[key] = model;
    }
    return map;
}

void KAMultiModelSettingPage::saveConfigToJson() {
    qDebug() << "saveConfigToJson";
    QJsonObject rootObj;
    rootObj["nlpModelMap"] = mapToJson(d_ptr->m_nlpModelMap);
    rootObj["visionModelMap"] = mapToJson(d_ptr->m_visionModelMap);
    rootObj["speechModelMap"] = mapToJson(d_ptr->m_speechModelMap);
    rootObj["nlpSelectedModel"] = d_ptr->nlpSelectedModel;
    rootObj["visionSelectedModel"] = d_ptr->visionSelectedModel;
    rootObj["speechSelectedModel"] = d_ptr->speechSelectedModel;
    rootObj["continuousSelected"] = d_ptr->continuousSelected;
    rootObj["realtimeSelected"] = d_ptr->realtimeSelected;

    QJsonDocument doc(rootObj);
    QString confJsonFile = QDir::homePath() + CONFIGPATH;
    QFile file(confJsonFile);
    if (file.open(QIODevice::WriteOnly)) {
        file.write(doc.toJson());
        file.close();
    } else {
        qDebug() << "Failed to open file for writing";
    }
}

void KAMultiModelSettingPage::loadConfigFromJson() {
    QString confJsonFile = QDir::homePath() + CONFIGPATH;
    QFile file(confJsonFile);
    if (!file.open(QIODevice::ReadOnly)) {
        qDebug() << "Failed to open file for reading";
        return;
    }

    QByteArray jsonData = file.readAll();
    file.close();

    QJsonDocument doc = QJsonDocument::fromJson(jsonData);
    if (!doc.isObject()) {
        qDebug() << "Invalid JSON document";
        return;
    }

    QJsonObject rootObj = doc.object();
    d_ptr->m_nlpModelMap = jsonToMap(rootObj["nlpModelMap"].toObject());
    d_ptr->m_visionModelMap = jsonToMap(rootObj["visionModelMap"].toObject());
    d_ptr->m_speechModelMap = jsonToMap(rootObj["speechModelMap"].toObject());
    d_ptr->nlpSelectedModel = rootObj["nlpSelectedModel"].toInt();
    d_ptr->visionSelectedModel = rootObj["visionSelectedModel"].toInt();
    d_ptr->speechSelectedModel = rootObj["speechSelectedModel"].toInt();
    d_ptr->continuousSelected = rootObj["continuousSelected"].toInt();
    d_ptr->realtimeSelected = rootObj["realtimeSelected"].toInt();

    reinitLayout();
}

/**
 * @brief KAMultiModelSettingPage::reinitLayout
 *          重新布局（模型有变化的时候重新布局）
 */
void KAMultiModelSettingPage::reinitLayout()
{
    // 清空modelmap所有控件
    d_ptr->m_nlpModelItemMap.clear();
    d_ptr->m_visionModelItemMap.clear();
    d_ptr->m_speechModelItemMap.clear();

    d_ptr->m_nlpModelGroup->removeAndDeleteAllWidget();
    d_ptr->m_visionModelGroup->removeAndDeleteAllWidget();
    d_ptr->m_speechModelGroup->removeAndDeleteAllWidget();
    d_ptr->m_nlpModelMapUI.clear();
    d_ptr->m_visionModelMapUI.clear();
    d_ptr->m_speechModelMapUI.clear();

    // addmodelitem会把SelectedModel改变，需要提前保存，用于设置默认选中状态
    ModelVendorId mid;
    mid = static_cast<ModelVendorId>(d_ptr->nlpSelectedModel);

    ModelVendorId mid2;
    mid2 = static_cast<ModelVendorId>(d_ptr->visionSelectedModel);

    ModelVendorId mid3;
    mid3 = static_cast<ModelVendorId>(d_ptr->speechSelectedModel);

    foreach (auto s_model, d_ptr->m_nlpModelMap) {
        addModelItem(s_model.modelTypeId, s_model.modelVendorId, false);   // false代表为首次loadConfigFromJson,无需保存
    }
    showModelItemSelection(NLP_TYPE, mid,false);

    foreach (auto s_model, d_ptr->m_visionModelMap) {
        addModelItem(s_model.modelTypeId, s_model.modelVendorId, false);
    }
    showModelItemSelection(VISION_TYPE, mid2,false);

    foreach (auto s_model, d_ptr->m_speechModelMap) {
        addModelItem(s_model.modelTypeId, s_model.modelVendorId, false);
    }
    showModelItemSelection(SPEECH_TYPE, mid3,false);
}


void KAMultiModelSettingPage::nlpAddModel()
{
   KAConfigModelSettingDialog settingDialog(this);
   settingDialog.initUIData(NLP_TYPE);
   settingDialog.exec();
}

void KAMultiModelSettingPage::visionAddModel()
{
    KAConfigModelSettingDialog settingDialog(this);
    settingDialog.initUIData(VISION_TYPE);
    settingDialog.exec();
}

void KAMultiModelSettingPage::speechAddModel()
{
    KAConfigModelSettingDialog settingDialog(this);
    settingDialog.initUIData(SPEECH_TYPE);
    settingDialog.exec();
}

void KAMultiModelSettingPage::initPrivateModelPage()
{
    // 私有化模型界面
    d_ptr->m_privateModelWidget = new QWidget;
    QVBoxLayout *privateModelLayout = new QVBoxLayout;
    QLabel *m_tip = new QLabel;
    m_tip->setText(tr("If you need to configure, please go to the Model Configuration Tool to do so.")); // 如需配置,请前往"模型配置工具"进行
    QFont font("Noto Sans CJK SC", 11);
    m_tip->setFont(font);
    privateModelLayout->addWidget(m_tip);
    privateModelLayout->addStretch();
    d_ptr->m_privateModelWidget->setLayout(privateModelLayout);
    d_ptr->m_contentLayout->addWidget(d_ptr->m_privateModelWidget);
    d_ptr->m_privateModelWidget->setVisible(false);
}

bool KAMultiModelSettingPage::eventFilter(QObject *watched, QEvent *event)
{
    Q_UNUSED(watched);

    ModelCardWidget *clickedModel = qobject_cast<ModelCardWidget*>(watched);

    if (clickedModel) {

        if (event->type() == QEvent::MouseButtonRelease) {

            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton) { // 判断左键

                resetModelColors();

                if (clickedModel == d_ptr->m_cloudModel || clickedModel == d_ptr->m_cloudModelHighlight) {
                    highlightClickedModel(d_ptr->m_cloudModel, d_ptr->m_cloudModelHighlight);
                    displayCloudModel();
                } else if (clickedModel == d_ptr->m_privateModel || clickedModel == d_ptr->m_privateModelHighlight) {
                    highlightClickedModel(d_ptr->m_privateModel, d_ptr->m_privateModelHighlight);
                    displyPrivateModel();
                } else if (clickedModel == d_ptr->m_localModel || clickedModel == d_ptr->m_localModelHighlight) {
                    highlightClickedModel(d_ptr->m_localModel, d_ptr->m_localModelHighlight);
                    displyLocalModel();
                // } else if (clickedModel == d_ptr->m_customModel || clickedModel == d_ptr->m_customModelHighlight) {
                //     highlightClickedModel(d_ptr->m_customModel, d_ptr->m_customModelHighlight);
                }
            }
        }
    }


    KAModelItem *clickedModelItem = qobject_cast<KAModelItem*>(watched);
    if (clickedModelItem) {
        if (event->type() == QEvent::MouseButtonRelease) {
            QMouseEvent *mouseEvent = static_cast<QMouseEvent *>(event);
            if (mouseEvent->button() == Qt::LeftButton) { // 判断左键
               showModelItemSelection(clickedModelItem->getModelTypeId(), clickedModelItem->getModelVendorId(), true);
               switchSelectCloudModel(clickedModelItem->getModelTypeId(), clickedModelItem->getModelVendorId());
            }
        }
    }

    return QWidget::eventFilter(watched,event);
}

void KAMultiModelSettingPage::resetModelColors()
{
    d_ptr->m_cloudModel->setVisible(true);
    d_ptr->m_privateModel->setVisible(true);
    d_ptr->m_localModel->setVisible(true);
    // d_ptr->m_customModel->setVisible(true);

    d_ptr->m_cloudModelHighlight->setVisible(false);
    d_ptr->m_privateModelHighlight->setVisible(false);
    d_ptr->m_localModelHighlight->setVisible(false);
    // d_ptr->m_customModelHighlight->setVisible(false);
}

void KAMultiModelSettingPage::highlightClickedModel(ModelCardWidget *baseModel, ModelCardWidget *highlightModel)
{
    highlightModel->setVisible(true);
    baseModel->setVisible(false);
}

void KAMultiModelSettingPage::showModelItemSelection(ModelTypeId modelTypeId, ModelVendorId modelVendorId, bool flag)
{
    QMap<ModelVendorId, KAModelItem*> *modelItemMap = nullptr;

    if (modelTypeId == NLP_TYPE) {
        modelItemMap = &d_ptr->m_nlpModelItemMap;
        d_ptr->nlpSelectedModel = modelVendorId;

    } else if (modelTypeId == VISION_TYPE) {
    modelItemMap = &d_ptr->m_visionModelItemMap;
    d_ptr->visionSelectedModel = modelVendorId;

    } else if (modelTypeId == SPEECH_TYPE) {
    modelItemMap = &d_ptr->m_speechModelItemMap;
    d_ptr->speechSelectedModel = modelVendorId;

    }
    if (modelItemMap) {
        foreach (const auto& modelItemId, modelItemMap->keys()) {
            KAModelItem* modelItem = modelItemMap->value(modelItemId);
            if (modelItem->getModelVendorId() == modelVendorId) {
                // 匹配成功，展示图标
                modelItem->showSelectedIcon(true);
                switchSelectCloudModel(modelItem->getModelTypeId(), modelItem->getModelVendorId());

            } else {
                // 不匹配，隐藏图标
                modelItem->showSelectedIcon(false);
            }
        }
    }
    if (flag) {
        saveConfigToJson();
    }

}

void KAMultiModelSettingPage::displayCloudModel()
{
    d_ptr->m_BigModelWidget->setVisible(true);
    d_ptr->m_nlpModelWidget->setVisible(true);
    d_ptr->m_visionModelWidget->setVisible(true);
    d_ptr->m_speechModelWidget->setVisible(true);
    d_ptr->m_privateModelWidget->setVisible(false);

    configCurrentCapabilityModelKeys();
    d_ptr->m_scrollContent->adjustSize();
}

void KAMultiModelSettingPage::displyPrivateModel()
{
    d_ptr->m_BigModelWidget->setVisible(false);
    d_ptr->m_nlpModelWidget->setVisible(false);
    d_ptr->m_visionModelWidget->setVisible(false);
    d_ptr->m_speechModelWidget->setVisible(false);
    d_ptr->m_privateModelWidget->setVisible(true);
    d_ptr->m_scrollContent->adjustSize();
    d_ptr->m_scrollArea->verticalScrollBar()->setVisible(false);
}

void KAMultiModelSettingPage::switchLocalModel()
{

    const char* json = R"(
           {
               "appId": "a",
               "apiKey": "b",
               "secretKey": "c"
           })";

    capability_settings_set_deploy_policy(CAPABILITY_NLP, LOCAL);
    capability_settings_set_model(CAPABILITY_NLP, LOCAL, "ondevice");
    capability_settings_set_model_config(CAPABILITY_NLP, LOCAL, "ondevice", json);

    capability_settings_set_deploy_policy(CAPABILITY_SPEECH, LOCAL);
    capability_settings_set_model(CAPABILITY_SPEECH, LOCAL, "ondevice");
    capability_settings_set_model_config(CAPABILITY_SPEECH, LOCAL, "ondevice", json);

    capability_settings_set_deploy_policy(CAPABILITY_VISION, LOCAL);
    capability_settings_set_model(CAPABILITY_VISION, LOCAL, "ondevice");
    capability_settings_set_model_config(CAPABILITY_VISION, LOCAL, "ondevice", json);

    g_gsettings->set("isNlpSetup", false);
    g_gsettings->set("isVisionSetup", false);
    g_gsettings->set("isSpeechSetup", false);
    g_gsettings->set("isNlpSetup", true);
    g_gsettings->set("isVisionSetup", true);
    g_gsettings->set("isSpeechSetup", true);

}

void KAMultiModelSettingPage::displyLocalModel()
{
    d_ptr->m_BigModelWidget->setVisible(false);
    d_ptr->m_nlpModelWidget->setVisible(false);
    d_ptr->m_visionModelWidget->setVisible(false);
    d_ptr->m_speechModelWidget->setVisible(false);
    d_ptr->m_privateModelWidget->setVisible(false);
    d_ptr->m_scrollContent->adjustSize();
    d_ptr->m_scrollArea->verticalScrollBar()->setVisible(false);
    switchLocalModel();
}

void KAMultiModelSettingPage::onChangeTheme()
{
    d_ptr->m_cloudModel->changeTheme();
    d_ptr->m_privateModel->changeTheme();
    d_ptr->m_localModel->changeTheme();
    // d_ptr->m_customModel->changeTheme();

    d_ptr->m_cloudModelHighlight->changeTheme();
    d_ptr->m_privateModelHighlight->changeTheme();
    d_ptr->m_localModelHighlight->changeTheme();
    // d_ptr->m_customModelHighlight->changeTheme();
}

void KAMultiModelSettingPage::addModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId, bool flag)
{
    UkccFrame *itemFrame = new UkccFrame;
    itemFrame->setMinimumWidth(496);
    itemFrame->setMaximumWidth(496);
    itemFrame->setFixedHeight(56);
    itemFrame->setObjectName(QString("aaa ").append(modelVendorId));
    QHBoxLayout *layItem = new QHBoxLayout(itemFrame);
    layItem->setContentsMargins(0, 0, 0, 0);

//    qDebug() << "______________" << modelTypeId << modelVendorId;

    KAModelItem *item = new KAModelItem(modelTypeId, modelVendorId);
    item->installEventFilter(this);
    connect(item, &KAModelItem::sigEdit, this, &KAMultiModelSettingPage::editModelItem);
    connect(item, &KAModelItem::sigDelete, this, &KAMultiModelSettingPage::deleteModelItem);
    layItem->addWidget(item);

    if (modelTypeId == NLP_TYPE) {
        d_ptr->m_nlpModelItemMap[modelVendorId] = item;
        d_ptr->m_nlpModelMapUI[modelVendorId] = itemFrame;
        d_ptr->m_nlpModelGroup->addWidget(itemFrame);
        if (d_ptr->m_nlpModelItemMap.count() == 1 && flag == true) {
            item->showSelectedIcon(true);
            if (flag) {
                switchSelectCloudModel(item->getModelTypeId(), item->getModelVendorId());
            }
            d_ptr->nlpSelectedModel = modelVendorId;
        }
        qDebug() << "addwidget";

    } else if (modelTypeId == VISION_TYPE) {
        d_ptr->m_visionModelItemMap[modelVendorId] = item;
        d_ptr->m_visionModelMapUI[modelVendorId] = itemFrame;
        d_ptr->m_visionModelGroup->addWidget(itemFrame);
        if (d_ptr->m_visionModelItemMap.count() == 1 && flag == true) {
            item->showSelectedIcon(true);
            if (flag) {
                switchSelectCloudModel(item->getModelTypeId(), item->getModelVendorId());
            }
            d_ptr->visionSelectedModel = modelVendorId;
        }

    } else if (modelTypeId == SPEECH_TYPE) {
        d_ptr->m_speechModelItemMap[modelVendorId] = item;
        d_ptr->m_speechModelMapUI[modelVendorId] = itemFrame;
        d_ptr->m_speechModelGroup->addWidget(itemFrame);
        if (d_ptr->m_speechModelItemMap.count() == 1 && flag == true) {
            item->showSelectedIcon(true);
            if (flag) {
                switchSelectCloudModel(item->getModelTypeId(), item->getModelVendorId());
            }
            d_ptr->speechSelectedModel = modelVendorId;
        }
    }

    if (flag) {
        saveConfigToJson();
    }

    // 控制滑动条的显示
    int itemCount = d_ptr->m_nlpModelItemMap.count() + d_ptr->m_visionModelItemMap.count() + d_ptr->m_speechModelItemMap.count();
    if(itemCount > 2) {
        d_ptr->m_scrollArea->verticalScrollBar()->setVisible(true);
    } else {
        d_ptr->m_scrollArea->verticalScrollBar()->setVisible(false);
    }
}

void KAMultiModelSettingPage::editModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId)
{
    if (modelTypeId == NLP_TYPE) {
        KAConfigModelSettingDialog settingDialog(d_ptr->m_nlpModelMap[modelVendorId], this);
        settingDialog.exec();

    } else if (modelTypeId == VISION_TYPE) {
        KAConfigModelSettingDialog settingDialog(d_ptr->m_visionModelMap[modelVendorId], this);
        settingDialog.exec();

    } else if (modelTypeId == SPEECH_TYPE) {
        KAConfigModelSettingDialog settingDialog(d_ptr->m_speechModelMap[modelVendorId], this);
        settingDialog.exec();

    }

}

bool KAMultiModelSettingPage::deleteModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId)
{
    if (modelTypeId == NLP_TYPE) {
        d_ptr->m_nlpModelMap.remove(modelVendorId);
        d_ptr->m_nlpModelItemMap.remove(modelVendorId);
        d_ptr->m_nlpModelGroup->removeWidget( d_ptr->m_nlpModelMapUI[modelVendorId]);
        d_ptr->m_nlpModelMapUI.remove(modelVendorId);

    } else if (modelTypeId == VISION_TYPE) {
        d_ptr->m_visionModelMap.remove(modelVendorId);
        d_ptr->m_visionModelItemMap.remove(modelVendorId);
        d_ptr->m_visionModelGroup->removeWidget(d_ptr->m_visionModelMapUI[modelVendorId]);
        d_ptr->m_visionModelMapUI.remove(modelVendorId);

    } else if (modelTypeId == SPEECH_TYPE) {
        d_ptr->m_speechModelMap.remove(modelVendorId);
        d_ptr->m_speechModelItemMap.remove(modelVendorId);
        d_ptr->m_speechModelGroup->removeWidget(d_ptr->m_speechModelMapUI[modelVendorId]);
        d_ptr->m_speechModelMapUI.remove(modelVendorId);

    }

    d_ptr->m_scrollContent->adjustSize();
    // 控制滑动条的显示
    int itemCount = d_ptr->m_nlpModelItemMap.count() + d_ptr->m_visionModelItemMap.count() + d_ptr->m_speechModelItemMap.count();
    if(itemCount > 2) {
        d_ptr->m_scrollArea->verticalScrollBar()->setVisible(true);
    } else {
        d_ptr->m_scrollArea->verticalScrollBar()->setVisible(false);
    }

    // 删除配置，需要设置gsetting相关属性值为false
    // 查询接口为 gsettings get org.kylin.aiassistant.settings is-vision-setup

    if (modelTypeId == NLP_TYPE) {
        if(d_ptr->nlpSelectedModel == modelVendorId) {
            g_gsettings->set("isNlpSetup", false);
            if (d_ptr->m_nlpModelItemMap.count() == 1) {        // 如果删除后只有一个模型，那么设置该模型为默认
                auto remainingModels = d_ptr->m_nlpModelItemMap.values();
                d_ptr->nlpSelectedModel = remainingModels.first()->getModelVendorId();
                remainingModels.first()->showSelectedIcon(true);
                switchSelectCloudModel(remainingModels.first()->getModelTypeId(), remainingModels.first()->getModelVendorId()); // set sdk

            } else if (d_ptr->m_nlpModelItemMap.count() == 0) { // 如果删除后无模型
                ModelVendorId modelVendorId = static_cast<ModelVendorId>(d_ptr->nlpSelectedModel);
                resetSelectCloudModel(NLP_TYPE, modelVendorId);
            }
        }


    } else if (modelTypeId == VISION_TYPE) {
        if(d_ptr->visionSelectedModel == modelVendorId) {
            g_gsettings->set("isVisionSetup", false);
            if (d_ptr->m_visionModelItemMap.count() == 1) {        // 如果删除后只有一个模型，那么设置该模型为默认
                auto remainingModels = d_ptr->m_visionModelItemMap.values();
                d_ptr->visionSelectedModel = remainingModels.first()->getModelVendorId();
                remainingModels.first()->showSelectedIcon(true);
                switchSelectCloudModel(remainingModels.first()->getModelTypeId(), remainingModels.first()->getModelVendorId()); // set sdk

            } else if (d_ptr->m_visionModelItemMap.count() == 0) { // 如果删除后无模型
                ModelVendorId modelVendorId = static_cast<ModelVendorId>(d_ptr->visionSelectedModel);
                resetSelectCloudModel(VISION_TYPE, modelVendorId);
            }
        }


    } else if (modelTypeId == SPEECH_TYPE) {
        if(d_ptr->speechSelectedModel == modelVendorId) {
            g_gsettings->set("isSpeechSetup", false);
            if (d_ptr->m_speechModelItemMap.count() == 1) {        // 如果删除后只有一个模型，那么设置该模型为默认
                auto remainingModels = d_ptr->m_speechModelItemMap.values();
                d_ptr->speechSelectedModel = remainingModels.first()->getModelVendorId();
                remainingModels.first()->showSelectedIcon(true);
                switchSelectCloudModel(remainingModels.first()->getModelTypeId(), remainingModels.first()->getModelVendorId()); // set sdk

            } else if (d_ptr->m_speechModelItemMap.count() == 0) { // 如果删除后无模型
                ModelVendorId modelVendorId = static_cast<ModelVendorId>(d_ptr->speechSelectedModel);
                resetSelectCloudModel(SPEECH_TYPE, modelVendorId);
            }
        }

    } else {

    }

    saveConfigToJson();
    return true;
}

const char * toModel(int i){
    switch(i){
        case 1:
            return "xunfei";
        case 0:
            return "baidu";
        default:
            return "";
    }
}

void KAMultiModelSettingPage::configCurrentCapabilityModelKeys()
{
    ModelVendorId modelVendorId;
    modelVendorId = static_cast<ModelVendorId>(d_ptr->nlpSelectedModel);
    switchSelectCloudModel(NLP_TYPE, modelVendorId);

    modelVendorId = static_cast<ModelVendorId>(d_ptr->visionSelectedModel);
    switchSelectCloudModel(VISION_TYPE, modelVendorId);

    modelVendorId = static_cast<ModelVendorId>(d_ptr->speechSelectedModel);
    switchSelectCloudModel(SPEECH_TYPE, modelVendorId);

//    const char *model;
//    model = toModel(d_ptr->nlpSelectedModel);
//    resetCurrentCapabilityModelKeys(CAPABILITY_NLP, model);
//    model = toModel(d_ptr->visionSelectedModel);
//    resetCurrentCapabilityModelKeys(CAPABILITY_VISION, model);
//    model = toModel(d_ptr->speechSelectedModel);
//    resetCurrentCapabilityModelKeys(CAPABILITY_SPEECH, model);

    g_gsettings->set("isNlpSetup", false);
    g_gsettings->set("isVisionSetup", false);
    g_gsettings->set("isSpeechSetup", false);
    g_gsettings->set("isNlpSetup", true);
    g_gsettings->set("isVisionSetup", true);
    g_gsettings->set("isSpeechSetup", true);
}

void KAMultiModelSettingPage::resetCurrentCapabilityModelKeys(Capability capability, const char *model)
{
    int modelLen = capability_settings_get_model_length(capability, capability_settings_get_deploy_policy(capability));
    if (modelLen <= 0) {
        return;
    }

    capability_settings_set_deploy_policy(capability, PUBLIC_CLOUD);

    capability_settings_set_model(capability,PUBLIC_CLOUD, model);

}

void KAMultiModelSettingPage::switchSelectCloudModel(ModelTypeId modelTypeId, ModelVendorId modelVendorId)
{
    Capability capability;
    const char* model;
    const char* json;
    QJsonObject obj;

    if (modelVendorId == 0) {
        model = "baidu";
    } else {
        model = "xunfei";
    }

    if (modelTypeId == 0) {
        capability = CAPABILITY_NLP;
        S_Model nlp_model = d_ptr->m_nlpModelMap.value(modelVendorId);
        obj.insert("appId", nlp_model.appId);
        obj.insert("apiKey", nlp_model.apiKey);
        obj.insert("secretKey", nlp_model.apiSecret);

    } else if(modelTypeId == 1) {
        capability = CAPABILITY_VISION;
        S_Model vision_model = d_ptr->m_visionModelMap.value(modelVendorId);
        obj.insert("appId", vision_model.appId);
        obj.insert("apiKey", vision_model.apiKey);
        obj.insert("secretKey", vision_model.apiSecret);

    } else if(modelTypeId == 2) {
        capability = CAPABILITY_SPEECH;
        S_Model speech_model = d_ptr->m_speechModelMap.value(modelVendorId);

        if (modelVendorId == 0) {
            obj.insert("appId", speech_model.appId);
            obj.insert("apiKey", speech_model.apiKey);
            obj.insert("secretKey", speech_model.apiSecret);
        } else {
            if (d_ptr->continuousSelected && d_ptr->realtimeSelected) {
                // 如果两个开关都被选中
                obj.insert("appId", speech_model.appIdContinuous);
                // obj.insert("appIdContinuous", speech_model.appIdContinuous);
                obj.insert("apiKeyContinuous", speech_model.apiKeyContinuous);
                obj.insert("secretKeyContinuous", speech_model.secretKeyContinuous);
                // obj.insert("appIdRealTime", speech_model.appIdRealTime);
                obj.insert("apiKeyRealTime", speech_model.apiKeyRealTime);
            } else if (d_ptr->continuousSelected) {
                // 只选择Continuous开关
                obj.insert("appId", speech_model.appIdContinuous);
                obj.insert("apiKeyContinuous", speech_model.apiKeyContinuous);
                obj.insert("secretKeyContinuous", speech_model.secretKeyContinuous);
                obj.insert("apiKeyRealTime", "");
            } else if (d_ptr->realtimeSelected) {
                // 只选择RealTime开关
                obj.insert("appId", speech_model.appIdRealTime);
                obj.insert("apiKeyContinuous", "");
                obj.insert("secretKeyContinuous", "");
                obj.insert("apiKeyRealTime", speech_model.apiKeyRealTime);
            }
        }
    } else {
        return;
    }
    QJsonDocument doc(obj);
    QString jsonString = doc.toJson(QJsonDocument::Indented);
    json = jsonString.toUtf8().constData();
//    qDebug() << json;
    capability_settings_set_deploy_policy(capability, PUBLIC_CLOUD);

    capability_settings_set_model(capability,PUBLIC_CLOUD, model);

    capability_settings_set_model_config(capability,
                                             DeployPolicy::PUBLIC_CLOUD,
                                             model, json);

    // 保存配置，如果NLP设置成功的话重新创建会话
    if (modelTypeId == NLP_TYPE) {
        g_gsettings->set("isNlpSetup", true);

    } else if (modelTypeId== VISION_TYPE) {
        // 发送信号告知已设置 vision 模型
        g_gsettings->set("isVisionSetup", true);
    } else if (modelTypeId== SPEECH_TYPE) {
        // 发送信号告知已设置 speech 模型
        g_gsettings->set("isSpeechSetup", true);
    }
}

void KAMultiModelSettingPage::resetSelectCloudModel(ModelTypeId modelTypeId, ModelVendorId modelVendorId)
{
    Capability capability;
    const char* model;

    if (modelVendorId == 0) {
        model = "baidu";
    } else {
        model = "xunfei";
    }

    if (modelTypeId == 0) {
        capability = CAPABILITY_NLP;


    } else if(modelTypeId == 1) {
        capability = CAPABILITY_VISION;


    } else if(modelTypeId == 2) {
        capability = CAPABILITY_SPEECH;

    } else {
        return;
    }

    // 重置某种AI具体能力的部署策略为默认值
    capability_settings_reset_deploy_policy(capability);

    // 重置某种AI具体能力使用的模型为默认值
    capability_settings_reset_model(capability, PUBLIC_CLOUD);

    // 重置某种AI具体能力使用的某种模型的配置为默认值
    capability_settings_reset_model_config(capability, PUBLIC_CLOUD, model);

}

class KAConfigModelSettingDialog::Private
{
public:
    S_Model model;
    bool isEdit = false;
    ModelTypeId vid;                                       // nlp vision speech

    QLabel *labelModelVendorType = nullptr;                // 模型类型
    QLabel *labelAPPID = nullptr;
    QLabel *labelApiKey = nullptr;
    QLabel *labelSecretKey = nullptr;
    QLabel *labelErrorInfo = nullptr;

    QComboBox *comboboxModelVendor = nullptr;              // baidu xunfei
//    QComboBox *comboboxSpeechType = nullptr;
    kdk::KPasswordEdit  *passwordAPPID = nullptr;
    kdk::KPasswordEdit  *passwordEditApi = nullptr;
    kdk::KPasswordEdit *passwordEditSecret = nullptr;

    // 流式
    QFrame *lineSeparator1 = nullptr;
    QLabel *labelSpeechContinuous = nullptr;
    QLabel *labelAPPIDContinuous = nullptr;
    QLabel *labelApiKeyContinuous = nullptr;
    QLabel *labelSecretKeyContinuous = nullptr;
    kdk::KSwitchButton *switchContinuous = nullptr;         // 开启：必填
    kdk::KPasswordEdit  *pwdAppIdContinuous = nullptr;
    kdk::KPasswordEdit  *pwdEditApiContinuous = nullptr;
    kdk::KPasswordEdit *pwdEditSecretContinuous = nullptr;
    // 实时
    QFrame *lineSeparator2 = nullptr;
    QLabel *labelSpeechRealTime = nullptr;
    QLabel *labelAPPIDRealTime = nullptr;
    QLabel *labelApiKeyRealTime = nullptr;
    kdk::KSwitchButton *switchRealTime = nullptr;           // 开启：必填
    kdk::KPasswordEdit  *pwdAppIdRealTime = nullptr;
    kdk::KPasswordEdit  *pwdEditApiRealTime = nullptr;

    QPushButton *buttonCancel = nullptr;
    QPushButton *buttonAccept = nullptr;

    bool isContinuousSelected = true;
    bool isRealTimeSelected = true;

    QMap<ModelVendorId, QVector<ModelTypeId>> mapAllModel;
};

KAConfigModelSettingDialog::KAConfigModelSettingDialog(S_Model model, QWidget *parent):
    KDialog{parent},
    d_ptr(make_shared<Private>())
{
    d_ptr->model = model;
    d_ptr->isEdit = true;
    initLayout();
    initConnect();
//    d_ptr->comboboxModelVendor->setCurrentIndex((int)model.modelVendorId);
//    d_ptr->comboboxModelVendor->setEnabled(false);
//    updateModelTypeCombobox();

//    setWindowTitle(tr("Edit model"));
}

KAConfigModelSettingDialog::KAConfigModelSettingDialog(QWidget *parent):
    KDialog{parent},
    d_ptr(make_shared<Private>())
{
    d_ptr->isEdit = false;
    initLayout();
    initConnect();

//    updateModelTypeCombobox();

//    setWindowTitle(tr("Add model"));
//    d_ptr->buttonAccept->setText(tr("Add"));

//    updatePasswordEdit(0);
//    updateXunfeiModel(0);
}


KAConfigModelSettingDialog::~KAConfigModelSettingDialog()
{

}

void KAConfigModelSettingDialog::initLayout()
{
    setWindowIcon(QIcon(":/icon/icon/AI-Writing.svg"));
    setFixedSize(420, 316);

    d_ptr->labelModelVendorType = new QLabel(tr("Model Type"));
    d_ptr->labelModelVendorType->setFixedWidth(80);
    d_ptr->labelAPPID = new QLabel(tr("APPID"));
    d_ptr->labelAPPID->setFixedWidth(80);
    d_ptr->labelApiKey = new QLabel("APIKey");
    d_ptr->labelApiKey->setFixedWidth(80);
    d_ptr->labelSecretKey = new QLabel("APISecret");
    d_ptr->labelSecretKey->setFixedWidth(80);

    d_ptr->comboboxModelVendor = new QComboBox;   //baidu,xunfei
    d_ptr->comboboxModelVendor->setFixedHeight(36);

    d_ptr->passwordAPPID = new kdk::KPasswordEdit(nullptr);
    d_ptr->passwordAPPID->setFocusPolicy(Qt::WheelFocus);
    d_ptr->passwordAPPID->setFixedHeight(36);
    d_ptr->passwordEditApi = new kdk::KPasswordEdit(nullptr);
    d_ptr->passwordEditApi->setFocusPolicy(Qt::WheelFocus);
    d_ptr->passwordEditApi->setFixedHeight(36);
    d_ptr->passwordEditSecret = new kdk::KPasswordEdit(nullptr);
    d_ptr->passwordEditSecret->setFocusPolicy(Qt::WheelFocus);
    d_ptr->passwordEditSecret->setFixedHeight(36);

    // 语音听写&在线语音合成(流式版)
    d_ptr->lineSeparator1 = new QFrame(this);
    d_ptr->lineSeparator1->setFrameShape(QFrame::HLine); // 横线
    d_ptr->lineSeparator1->setFixedHeight(1);

    d_ptr->labelSpeechContinuous = new QLabel(tr("Voice dictation&online voice synthesis (streaming version)"));
    d_ptr->labelAPPIDContinuous = new QLabel(tr("APPID"));
    d_ptr->labelAPPIDContinuous->setFixedWidth(80);
    d_ptr->labelApiKeyContinuous = new QLabel("APIKey");
    d_ptr->labelApiKeyContinuous->setFixedWidth(80);
    d_ptr->labelSecretKeyContinuous = new QLabel("APISecret");
    d_ptr->labelSecretKeyContinuous->setFixedWidth(80);

    d_ptr->switchContinuous = new kdk::KSwitchButton(nullptr);
    d_ptr->switchContinuous->setChecked(true);
    d_ptr->switchContinuous->setFixedSize(44, 24);

    d_ptr->pwdAppIdContinuous = new kdk::KPasswordEdit(nullptr);
    d_ptr->pwdAppIdContinuous->setFocusPolicy(Qt::WheelFocus);
    d_ptr->pwdAppIdContinuous->setFixedHeight(36);
    d_ptr->pwdEditApiContinuous = new kdk::KPasswordEdit(nullptr);
    d_ptr->pwdEditApiContinuous->setFocusPolicy(Qt::WheelFocus);
    d_ptr->pwdAppIdContinuous->setFixedHeight(36);
    d_ptr->pwdEditSecretContinuous = new kdk::KPasswordEdit(nullptr);
    d_ptr->pwdEditSecretContinuous->setFocusPolicy(Qt::WheelFocus);
    d_ptr->pwdAppIdContinuous->setFixedHeight(36);

    // 实时语音转写
    d_ptr->lineSeparator2 = new QFrame(this);
    d_ptr->lineSeparator2->setFrameShape(QFrame::HLine); // 横线
    d_ptr->lineSeparator2->setFixedHeight(1);

    d_ptr->labelSpeechRealTime = new QLabel(tr("Real time voice transcription"));
    d_ptr->labelAPPIDRealTime = new QLabel(tr("APPID"));
    d_ptr->labelAPPIDRealTime->setFixedWidth(80);
    d_ptr->labelApiKeyRealTime = new QLabel("APIKey");
    d_ptr->labelApiKeyRealTime->setFixedWidth(80);

    d_ptr->switchRealTime = new kdk::KSwitchButton(nullptr);
    d_ptr->switchRealTime->setChecked(true);
    d_ptr->switchRealTime->setFixedSize(44, 24);

    d_ptr->pwdAppIdRealTime = new kdk::KPasswordEdit(nullptr);
    d_ptr->pwdAppIdRealTime->setFocusPolicy(Qt::WheelFocus);
    d_ptr->pwdAppIdRealTime->setFixedHeight(36);
    d_ptr->pwdEditApiRealTime = new kdk::KPasswordEdit(nullptr);
    d_ptr->pwdEditApiRealTime->setFocusPolicy(Qt::WheelFocus);
    d_ptr->pwdEditApiRealTime->setFixedHeight(36);

    QGridLayout *gridLayout = new QGridLayout();
    gridLayout->addWidget(d_ptr->labelModelVendorType, 0, 0);            // 第一行第一列
    gridLayout->addWidget(d_ptr->labelAPPID, 1, 0);
    gridLayout->addWidget(d_ptr->labelApiKey, 2, 0);
    gridLayout->addWidget(d_ptr->labelSecretKey, 3, 0);

    gridLayout->addWidget(d_ptr->comboboxModelVendor, 0, 1);             // 第一行第二列
    gridLayout->addWidget(d_ptr->passwordAPPID, 1, 1);
    gridLayout->addWidget(d_ptr->passwordEditApi, 2, 1);
    gridLayout->addWidget(d_ptr->passwordEditSecret, 3, 1);

    // 语音听写&在线语音合成(流式版)
    QHBoxLayout *continuousLayout = new QHBoxLayout;
    continuousLayout->setContentsMargins(0, 0, 0, 0);
    d_ptr->labelSpeechContinuous->setAlignment(Qt::AlignVCenter);
    continuousLayout->addWidget(d_ptr->labelSpeechContinuous);
    continuousLayout->addStretch();
    continuousLayout->addWidget(d_ptr->switchContinuous);

    QGridLayout *gridContinuousLayout = new QGridLayout();
    gridContinuousLayout->addWidget(d_ptr->labelAPPIDContinuous, 0, 0);  // 第一行第一列
    gridContinuousLayout->addWidget(d_ptr->labelApiKeyContinuous, 1, 0);
    gridContinuousLayout->addWidget(d_ptr->labelSecretKeyContinuous, 2, 0);

    gridContinuousLayout->addWidget(d_ptr->pwdAppIdContinuous, 0, 1);    // 第一行第二列
    gridContinuousLayout->addWidget(d_ptr->pwdEditApiContinuous, 1, 1);
    gridContinuousLayout->addWidget(d_ptr->pwdEditSecretContinuous, 2, 1);

    d_ptr->lineSeparator1->setVisible(false);
    d_ptr->labelSpeechContinuous->setVisible(false);
    d_ptr->labelAPPIDContinuous->setVisible(false);
    d_ptr->labelApiKeyContinuous->setVisible(false);
    d_ptr->labelSecretKeyContinuous->setVisible(false);
    d_ptr->switchContinuous->setVisible(false);
    d_ptr->pwdAppIdContinuous->setVisible(false);
    d_ptr->pwdEditApiContinuous->setVisible(false);
    d_ptr->pwdEditSecretContinuous->setVisible(false);

    // 实时语音转写
    QHBoxLayout *realTimeLayout = new QHBoxLayout;
    realTimeLayout->setContentsMargins(0, 0, 0, 0);
    d_ptr->labelSpeechRealTime->setAlignment(Qt::AlignVCenter);
    realTimeLayout->addWidget(d_ptr->labelSpeechRealTime);
    realTimeLayout->addStretch();
    realTimeLayout->addWidget(d_ptr->switchRealTime);

    QGridLayout *gridRealTimeLayout = new QGridLayout();
    gridRealTimeLayout->addWidget(d_ptr->labelAPPIDRealTime, 0, 0);      // 第一行第一列
    gridRealTimeLayout->addWidget(d_ptr->labelApiKeyRealTime, 1, 0);

    gridRealTimeLayout->addWidget(d_ptr->pwdAppIdRealTime, 0, 1);        // 第一行第二列
    gridRealTimeLayout->addWidget(d_ptr->pwdEditApiRealTime, 1, 1);

    d_ptr->lineSeparator2->setVisible(false);
    d_ptr->labelSpeechRealTime->setVisible(false);
    d_ptr->labelAPPIDRealTime->setVisible(false);
    d_ptr->labelApiKeyRealTime->setVisible(false);
    d_ptr->switchRealTime->setVisible(false);
    d_ptr->pwdAppIdRealTime->setVisible(false);
    d_ptr->pwdEditApiRealTime->setVisible(false);

    d_ptr->labelErrorInfo = new QLabel();
    d_ptr->labelErrorInfo->setStyleSheet("color:red");
    d_ptr->labelErrorInfo->setFixedHeight(20);

    QHBoxLayout *errorInfoLayout = new QHBoxLayout;
    errorInfoLayout->setContentsMargins(90, 0, 0, 0);
    d_ptr->labelErrorInfo->setAlignment(Qt::AlignVCenter);
    errorInfoLayout->addWidget(d_ptr->labelErrorInfo);
    errorInfoLayout->addStretch();

    d_ptr->buttonCancel = new QPushButton(tr("Cancel"));
    d_ptr->buttonCancel->setFixedHeight(36);
    d_ptr->buttonCancel->setFocusPolicy(Qt::NoFocus);
    d_ptr->buttonCancel->setProperty("useButtonPalette", true);

    d_ptr->buttonAccept = new QPushButton(tr("Accept"));
    d_ptr->buttonAccept->setFixedHeight(36);
    d_ptr->buttonAccept->setFocusPolicy(Qt::NoFocus);
    d_ptr->buttonAccept->setEnabled(false);

    QHBoxLayout *buttonLayout = new QHBoxLayout;
    buttonLayout->setContentsMargins(0, 4, 0, 0);
    buttonLayout->addStretch();
    buttonLayout->addWidget(d_ptr->buttonCancel);
    buttonLayout->addWidget(d_ptr->buttonAccept);

    QVBoxLayout *mainLayout = new QVBoxLayout(mainWidget());
    mainLayout->setContentsMargins(24, 16, 24, 24);
    mainLayout->addLayout(gridLayout);
    mainLayout->addWidget(d_ptr->lineSeparator1);
    mainLayout->addLayout(continuousLayout);
    mainLayout->addLayout(gridContinuousLayout);
    mainLayout->addWidget(d_ptr->lineSeparator2);
    mainLayout->addLayout(realTimeLayout);
    mainLayout->addLayout(gridRealTimeLayout);
    mainLayout->addLayout(errorInfoLayout);
    mainLayout->addLayout(buttonLayout);
}

void KAConfigModelSettingDialog::initConnect()
{
    connect(d_ptr->comboboxModelVendor, SIGNAL(currentIndexChanged(int)), this, SLOT(updatePasswordEdit(int)));
    connect(d_ptr->buttonCancel, &QPushButton::clicked, [this](){reject();});
    connect(d_ptr->buttonAccept, &QPushButton::clicked, [this](){
        accept();
    });
    connect(d_ptr->switchContinuous, &KSwitchButton::stateChanged, this, &KAConfigModelSettingDialog::onSwitchContinuousChanged);
    connect(d_ptr->switchRealTime, &KSwitchButton::stateChanged, this, &KAConfigModelSettingDialog::onSwitchRealTimeChanged);

    connect(d_ptr->passwordAPPID, &KPasswordEdit::textChanged, this, [this]() { updateButtonStateEditing(d_ptr->passwordAPPID); });
    connect(d_ptr->passwordEditApi, &KPasswordEdit::textChanged, this, [this]() { updateButtonStateEditing(d_ptr->passwordEditApi); });
    connect(d_ptr->passwordEditSecret, &KPasswordEdit::textChanged, this, [this]() { updateButtonStateEditing(d_ptr->passwordEditSecret); });

    connect(d_ptr->pwdAppIdContinuous, &KPasswordEdit::textChanged, this, [this]() { updateButtonStateEditing(d_ptr->passwordAPPID); });
    connect(d_ptr->pwdEditApiContinuous, &KPasswordEdit::textChanged, this, [this]() { updateButtonStateEditing(d_ptr->passwordEditApi); });
    connect(d_ptr->pwdEditSecretContinuous, &KPasswordEdit::textChanged, this, [this]() { updateButtonStateEditing(d_ptr->passwordEditSecret); });
    connect(d_ptr->pwdAppIdRealTime, &KPasswordEdit::textChanged, this, [this]() { updateButtonStateEditing(d_ptr->passwordAPPID); });
    connect(d_ptr->pwdEditApiRealTime, &KPasswordEdit::textChanged, this, [this]() { updateButtonStateEditing(d_ptr->passwordEditApi); });

}

void KAConfigModelSettingDialog::updatePasswordEdit(int index)
{
    if (parentWidget())
    {
        this->setFixedSize(420, 316);
        KAMultiModelSettingPage *modelSettingPage = static_cast<KAMultiModelSettingPage*> (parentWidget());
        if (d_ptr->isEdit == true){
            if (d_ptr->vid == NLP_TYPE) {
                if (modelSettingPage->d_ptr->m_nlpModelMap.find((ModelVendorId)index) != modelSettingPage->d_ptr->m_visionModelMap.end()) {
                    setWindowTitle(tr("Edit model"));
                    d_ptr->buttonAccept->setText(tr("Accept"));

                    S_Model model = modelSettingPage->d_ptr->m_nlpModelMap[(ModelVendorId)index];
                    d_ptr->passwordAPPID->setEchoMode(QLineEdit::Normal);
                    d_ptr->passwordAPPID->setEchoModeBtnVisible(false);
                    d_ptr->passwordEditApi->setEchoMode(QLineEdit::Normal);
                    d_ptr->passwordEditApi->setEchoModeBtnVisible(false);
                    d_ptr->passwordEditSecret->setEchoMode(QLineEdit::Normal);
                    d_ptr->passwordEditSecret->setEchoModeBtnVisible(false);
                    QString str = model.appId.left(4) + "****" + model.appId.right(4);
                    d_ptr->passwordAPPID->setText(str);
                    str.clear();
                    str = model.apiKey.left(4) + "****" + model.apiKey.right(4);
                    d_ptr->passwordEditApi->setText(str);
                    str.clear();
                    str = model.apiSecret.left(4) + "****" + model.apiSecret.right(4);
                    d_ptr->passwordEditSecret->setText(str);
                }
            } else if (d_ptr->vid == VISION_TYPE) {
                if (modelSettingPage->d_ptr->m_visionModelMap.find((ModelVendorId)index) != modelSettingPage->d_ptr->m_visionModelMap.end()) {
                    setWindowTitle(tr("Edit model"));
                    d_ptr->buttonAccept->setText(tr("Accept"));

                    S_Model model = modelSettingPage->d_ptr->m_visionModelMap[(ModelVendorId)index];
                    d_ptr->passwordAPPID->setEchoMode(QLineEdit::Normal);
                    d_ptr->passwordAPPID->setEchoModeBtnVisible(false);
                    d_ptr->passwordEditApi->setEchoMode(QLineEdit::Normal);
                    d_ptr->passwordEditApi->setEchoModeBtnVisible(false);
                    d_ptr->passwordEditSecret->setEchoMode(QLineEdit::Normal);
                    d_ptr->passwordEditSecret->setEchoModeBtnVisible(false);
                    QString str = model.appId.left(4) + "****" + model.appId.right(4);
                    d_ptr->passwordAPPID->setText(str);
                    str.clear();
                    str = model.apiKey.left(4) + "****" + model.apiKey.right(4);
                    d_ptr->passwordEditApi->setText(str);
                    str.clear();
                    str = model.apiSecret.left(4) + "****" + model.apiSecret.right(4);
                    d_ptr->passwordEditSecret->setText(str);
                }
            } else if (d_ptr->vid == SPEECH_TYPE) {
//                if (modelSettingPage->d_ptr->m_speechModelMap.find((ModelVendorId)index) != modelSettingPage->d_ptr->m_speechModelMap.end()) {
//                    setWindowTitle(tr("Edit model"));
//                    d_ptr->buttonAccept->setText(tr("Accept"));

//                    S_Model model = modelSettingPage->d_ptr->m_speechModelMap[(ModelVendorId)index];
//                    d_ptr->passwordAPPID->setEchoMode(QLineEdit::Normal);
//                    d_ptr->passwordAPPID->setEchoModeBtnVisible(false);
//                    d_ptr->passwordEditApi->setEchoMode(QLineEdit::Normal);
//                    d_ptr->passwordEditApi->setEchoModeBtnVisible(false);
//                    d_ptr->passwordEditSecret->setEchoMode(QLineEdit::Normal);
//                    d_ptr->passwordEditSecret->setEchoModeBtnVisible(false);
//                    QString str = model.appId.left(4) + "****" + model.appId.right(4);
//                    d_ptr->passwordAPPID->setText(str);
//                    str.clear();
//                    str = model.apiKey.left(4) + "****" + model.apiKey.right(4);
//                    d_ptr->passwordEditApi->setText(str);
//                    str.clear();
//                    str = model.apiSecret.left(4) + "****" + model.apiSecret.right(4);
//                    d_ptr->passwordEditSecret->setText(str);
//                }
            }
        } else {
            // 添加时
            d_ptr->buttonAccept->setText(tr("Add"));
            d_ptr->passwordAPPID->setText("");
            d_ptr->passwordEditApi->setText("");
            d_ptr->passwordEditSecret->setText("");
            d_ptr->labelErrorInfo->setText("");
        }

        // 迅飞语音大模型
        if (d_ptr->vid == SPEECH_TYPE && index == 1) {
            this->setFixedSize(420, 502);
            // 隐藏
            d_ptr->labelAPPID->setVisible(false);
            d_ptr->labelApiKey->setVisible(false);
            d_ptr->labelSecretKey->setVisible(false);
            d_ptr->passwordAPPID->setVisible(false);
            d_ptr->passwordEditApi->setVisible(false);
            d_ptr->passwordEditSecret->setVisible(false);
            // 显示
            d_ptr->lineSeparator1->setVisible(true);
            d_ptr->labelSpeechContinuous->setVisible(true);
            d_ptr->labelAPPIDContinuous->setVisible(true);
            d_ptr->labelApiKeyContinuous->setVisible(true);
            d_ptr->labelSecretKeyContinuous->setVisible(true);
            d_ptr->switchContinuous->setVisible(true);
            d_ptr->pwdAppIdContinuous->setVisible(true);
            d_ptr->pwdEditApiContinuous->setVisible(true);
            d_ptr->pwdEditSecretContinuous->setVisible(true);

            d_ptr->lineSeparator2->setVisible(true);
            d_ptr->labelSpeechRealTime->setVisible(true);
            d_ptr->labelAPPIDRealTime->setVisible(true);
            d_ptr->labelApiKeyRealTime->setVisible(true);
            d_ptr->switchRealTime->setVisible(true);
            d_ptr->pwdAppIdRealTime->setVisible(true);
            d_ptr->pwdEditApiRealTime->setVisible(true);
        }
    }
}

QString KAConfigModelSettingDialog::modelVendorToString(const ModelTypeId modelTypeId, const ModelVendorId modelVendorId)
{
    if (modelTypeId == NLP_TYPE) {
        switch (modelVendorId)
        {
            case BAIDU_TYPE:
                return tr("BAIDU-ERNIE 4.0");  //百度-ERNIE 4.0
            case XUNFEI_TYPE:
                return tr("XUNFEI-Starfire Model V3.5"); //迅飞-星火大模型 V3.5
            default:
                return tr("No Vendor");
        }
    } else if (modelTypeId == VISION_TYPE) {
        switch (modelVendorId)
        {
            case BAIDU_TYPE:
                return tr("BAIDU-AI Painting-Advanced Edition"); //百度-AI作画-高级版
            case XUNFEI_TYPE:
                return tr("XUNFEI--Image Generation"); //迅飞--图片生成
            default:
                return tr("No Vendor");
        }
    } else if (modelTypeId == SPEECH_TYPE) {
        switch (modelVendorId)
        {
            case BAIDU_TYPE:
                return tr("BAIDU-Voice Big Model"); //百度-语音大模型
            case XUNFEI_TYPE:
                return tr("XUNFEI-Voice Big Model"); //迅飞-语音大模型
            default:
                return tr("No Vendor");
        }
    }
    return tr("No Vendor");
}

void KAConfigModelSettingDialog::switchButtonState()
{
    if (d_ptr->passwordEditSecret->isVisible()) {
        if (d_ptr->passwordAPPID->text().length() > 0 && d_ptr->passwordEditApi->text().length()>= 12 && d_ptr->passwordEditSecret->text().length() >= 12)
        {
            d_ptr->buttonAccept->setEnabled(true);
            d_ptr->buttonAccept->setProperty("isImportant", true);

        } else {
            d_ptr->buttonAccept->setEnabled(false);
        }
    } else {
        if (d_ptr->passwordAPPID->text().length() > 0 && d_ptr->passwordEditApi->text().length()>= 12)
        {
            d_ptr->buttonAccept->setEnabled(true);
            d_ptr->buttonAccept->setProperty("isImportant", true);

        } else {
            d_ptr->buttonAccept->setEnabled(false);
        }
    }

    // 在已选择的没填完之前，置灰；在无选择的时候，置灰

    if (d_ptr->switchContinuous->isVisible() && d_ptr->switchRealTime->isVisible())
    {
        bool bothSwitchesChecked = d_ptr->switchRealTime->isChecked() && d_ptr->switchContinuous->isChecked();
        bool realTimeChecked = d_ptr->switchRealTime->isChecked();
        bool continuousChecked = d_ptr->switchContinuous->isChecked();
        bool enableButton = false;
        bool isImportant = false;
        // 如果两个开关都被选择
        if (bothSwitchesChecked) {
            if (d_ptr->pwdAppIdContinuous->text().length() > 0
                && d_ptr->pwdAppIdRealTime->text().length() > 0
                && d_ptr->pwdEditApiRealTime->text().length() >= 12
                && d_ptr->pwdEditApiContinuous->text().length() >= 12
                && d_ptr->pwdEditSecretContinuous->text().length() >= 12) {
                enableButton = true;
                isImportant = true;
            }
        }
        // 如果只有一个开关被选择
        else if (realTimeChecked || continuousChecked) {
            if (realTimeChecked
                && d_ptr->pwdAppIdRealTime->text().length() > 0
                && d_ptr->pwdEditApiRealTime->text().length() >= 12) {
                enableButton = true;
                isImportant = true;

            } else if (continuousChecked
                       && d_ptr->pwdAppIdContinuous->text().length() > 0
                       && d_ptr->pwdEditApiContinuous->text().length() >= 12
                       && d_ptr->pwdEditSecretContinuous->text().length() >= 12) {
                enableButton = true;
                isImportant = true;
            }
        }
        // 没有开关被选择
        else {
            enableButton = false;
            isImportant = false;
        }

        d_ptr->buttonAccept->setEnabled(enableButton);
        d_ptr->buttonAccept->setProperty("isImportant", isImportant);
    }
}

void KAConfigModelSettingDialog::updateButtonStateEditing(KPasswordEdit *passwordEdit)
{
    if (passwordEdit->text().length() == 0) {
        passwordEdit->setEchoModeBtnVisible(true);
    }
    switchButtonState();
}

void KAConfigModelSettingDialog::onSwitchContinuousChanged(bool isVisible) {
    d_ptr->labelAPPIDContinuous->setVisible(isVisible);
    d_ptr->labelApiKeyContinuous->setVisible(isVisible);
    d_ptr->labelSecretKeyContinuous->setVisible(isVisible);
    d_ptr->pwdAppIdContinuous->setVisible(isVisible);
    d_ptr->pwdEditApiContinuous->setVisible(isVisible);
    d_ptr->pwdEditSecretContinuous->setVisible(isVisible);

    adjustDialogSize();
    switchButtonState();
    d_ptr->isContinuousSelected = isVisible;
}

void KAConfigModelSettingDialog::onSwitchRealTimeChanged(bool isVisible) {
    d_ptr->labelAPPIDRealTime->setVisible(isVisible);
    d_ptr->labelApiKeyRealTime->setVisible(isVisible);
    d_ptr->pwdAppIdRealTime->setVisible(isVisible);
    d_ptr->pwdEditApiRealTime->setVisible(isVisible);

    adjustDialogSize();
    switchButtonState();
    d_ptr->isRealTimeSelected = isVisible;
}

void KAConfigModelSettingDialog::adjustDialogSize() {
    if (d_ptr->switchContinuous->isVisible() && d_ptr->switchRealTime->isVisible())
    {
        bool bothSwitchesChecked = d_ptr->switchRealTime->isChecked() && d_ptr->switchContinuous->isChecked();
        bool realTimeChecked = d_ptr->switchRealTime->isChecked();
        bool continuousChecked = d_ptr->switchContinuous->isChecked();

        // 如果两个开关都被选择
        if (bothSwitchesChecked) {
            this->setFixedSize(420, 502);
        }
        // 如果只有一个开关被选择
        else if (realTimeChecked || continuousChecked) {
            this->setFixedSize(420, 414);
        }
        // 没有开关被选择
        else {
            this->setFixedSize(420, 316);
        }
    }
}

/**
 * @brief KAConfigModelSettingDialog::initUIData
 *          初始化所有模型数据
 */
void KAConfigModelSettingDialog::initUIData(const ModelTypeId vid)
{
    d_ptr->vid = vid;
    if(vid == NLP_TYPE){
        setWindowTitle(tr("Add-NLP Language Big Model")); //添加-NLP语言大模型
    }else if(vid == VISION_TYPE){
        setWindowTitle(tr("Add-VISION Visual Model")); //添加-vision视觉大模型
    }else if(vid == SPEECH_TYPE){
        setWindowTitle(tr("Add-SPEECH Speech Model")); //添加-SPEECH语音大模型
    }

    for (int vendor = BAIDU_TYPE; vendor <= XUNFEI_TYPE; ++vendor)
    {
        d_ptr->comboboxModelVendor->addItem(modelVendorToString(static_cast<ModelTypeId>(vid), static_cast<ModelVendorId>(vendor)));
    }

    QString passwdTip = tr("Must write");
    d_ptr->passwordAPPID->setPlaceholderText(passwdTip);
    d_ptr->passwordEditApi->setPlaceholderText(passwdTip);
    d_ptr->passwordEditSecret->setPlaceholderText(passwdTip);

    d_ptr->pwdAppIdContinuous->setPlaceholderText(passwdTip);
    d_ptr->pwdEditApiContinuous->setPlaceholderText(passwdTip);
    d_ptr->pwdEditSecretContinuous->setPlaceholderText(passwdTip);
    d_ptr->pwdAppIdRealTime->setPlaceholderText(passwdTip);
    d_ptr->pwdEditApiRealTime->setPlaceholderText(passwdTip);
}

void KAConfigModelSettingDialog::accept()
{
    //判断是否存在该模型
    KAMultiModelSettingPage *modelSettingPage = nullptr;
    modelSettingPage = static_cast<KAMultiModelSettingPage*> (parentWidget());
    if (d_ptr->isEdit == false){
        if (parentWidget()){
            // 判断是否存在该模型
            if (d_ptr->vid == NLP_TYPE) {
                if (modelSettingPage->d_ptr->m_nlpModelItemMap.contains((ModelVendorId)(d_ptr->comboboxModelVendor->currentIndex()))) {
                    qDebug() << "nlp" << d_ptr->vid  << "already exist";
                    d_ptr->labelErrorInfo->setText(tr("The model type already exists. Cannot be repeated!"));
                    return;
                }
            } else if (d_ptr->vid == VISION_TYPE) {
                if (modelSettingPage->d_ptr->m_visionModelItemMap.contains((ModelVendorId)(d_ptr->comboboxModelVendor->currentIndex()))) {
                    qDebug() << "vision" << d_ptr->vid  << "already exist";
                    d_ptr->labelErrorInfo->setText(tr("The model type already exists. Cannot be repeated!"));
                    return;
                }
            } else if (d_ptr->vid == SPEECH_TYPE) {
                if (modelSettingPage->d_ptr->m_speechModelItemMap.contains((ModelVendorId)(d_ptr->comboboxModelVendor->currentIndex()))) {
                    qDebug() << "speech" << d_ptr->vid  << "already exist";
                    d_ptr->labelErrorInfo->setText(tr("The model type already exists. Cannot be repeated!"));
                    return;
                }
            }
        }
    }

    // 添加到对应的modelmap
    if (parentWidget())
    {
        S_Model model;
        model.modelTypeId = d_ptr->vid;
        model.modelVendorId = (ModelVendorId)(d_ptr->comboboxModelVendor->currentIndex());

        if (d_ptr->switchContinuous->isVisible() && d_ptr->switchRealTime->isVisible())
        {
            model.appId = "";
            model.apiKey = "";
            model.apiSecret = "";

            model.appIdContinuous = d_ptr->pwdAppIdContinuous->text();
            model.apiKeyContinuous = d_ptr->pwdEditApiContinuous->text();
            model.secretKeyContinuous = d_ptr->pwdEditSecretContinuous->text();

            model.appIdRealTime = d_ptr->pwdAppIdRealTime->text();
            model.apiKeyRealTime = d_ptr->pwdEditApiRealTime->text();
        } else {                                                     // 其他
            model.appId = d_ptr->passwordAPPID->text();
            model.apiKey = d_ptr->passwordEditApi->text();
            model.apiSecret = d_ptr->passwordEditSecret->text();

            model.appIdContinuous = "";
            model.apiKeyContinuous = "";
            model.secretKeyContinuous = "";
            model.appIdRealTime = "";
            model.apiKeyRealTime = "";
        }

        if (parentWidget()){
            if (d_ptr->vid == NLP_TYPE) {
                modelSettingPage->d_ptr->m_nlpModelMap[model.modelVendorId] = model;

            } else if (d_ptr->vid == VISION_TYPE) {
                modelSettingPage->d_ptr->m_visionModelMap[model.modelVendorId] = model;

            } else if (d_ptr->vid == SPEECH_TYPE) {
                modelSettingPage->d_ptr->m_speechModelMap[model.modelVendorId] = model;
                // 同步两个开关的状态
                modelSettingPage->d_ptr->continuousSelected = d_ptr->isContinuousSelected;
                modelSettingPage->d_ptr->realtimeSelected = d_ptr->isRealTimeSelected;
            }
        }

        modelSettingPage->addModelItem(d_ptr->vid, model.modelVendorId, true);
        modelSettingPage->reinitLayout();
    }

    d_ptr->labelErrorInfo->setText("");
    return KDialog::accept();
}
