#ifndef KAMULTIMODELSETTINGPAGE_H
#define KAMULTIMODELSETTINGPAGE_H

#include <QWidget>
#include <memory>
#include <QJsonObject>
#include <QJsonDocument>
#include <QJsonArray>
#include <QFile>
#include <QDir>

#include "modelcardwidget.h"
#include "kamodelitem.h"
#include <kdialog.h>
#include <kylin-ai/config.h>
#include <kysdk/applications/kpasswordedit.h>

using namespace std;
using namespace kdk;

//typedef enum { NLP_TYPE, VISION_TYPE, SPEECH_TYPE} ModelTypeId;
//typedef enum { BAIDU_TYPE, XUNFEI_TYPE} ModelVendorId;

typedef struct {
    ModelTypeId modelTypeId;
    ModelVendorId modelVendorId;
    QString appId;
    QString apiKey;
    QString apiSecret;

    QString appIdContinuous;
    QString apiKeyContinuous;
    QString secretKeyContinuous;

    QString appIdRealTime;
    QString apiKeyRealTime;
//    QString secretKeyRealTime;
} S_Model;

class KAConfigModelSettingDialog : public KDialog
{
    Q_OBJECT
public:
    explicit KAConfigModelSettingDialog(S_Model model, QWidget *parent = nullptr);
    explicit KAConfigModelSettingDialog(QWidget *parent = nullptr);
    ~KAConfigModelSettingDialog();
    void initUIData(const ModelTypeId vid);

signals:

public slots:


private:
    void initLayout();
    void initConnect();
    void adjustDialogSize();

    QString modelVendorToString(const ModelTypeId modelTypeId, const ModelVendorId modelVendorId);

    void accept() override;

private slots:
    void updatePasswordEdit(int index);
    void switchButtonState();
    void updateButtonStateEditing(KPasswordEdit *passwordEdit);
    void onSwitchContinuousChanged(bool isVisible);
    void onSwitchRealTimeChanged(bool isVisible);

private:
    class Private;
    shared_ptr<Private> d_ptr;
};


class KAMultiModelSettingPage : public QWidget
{
    Q_OBJECT
    friend class KAConfigModelSettingDialog;
public:
    explicit KAMultiModelSettingPage(QWidget *parent = nullptr);
    ~KAMultiModelSettingPage();

signals:


public slots:
    void onChangeTheme();
    void editModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId);
    bool deleteModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId);

protected:
    bool eventFilter(QObject *watched, QEvent *event) override;


private slots:
    void nlpAddModel();
    void visionAddModel();
    void speechAddModel();


private:
    void initLayout();
    void reinitLayout();
    void resetModelColors();
    void highlightClickedModel(ModelCardWidget *baseModel, ModelCardWidget *highlightModel);
    void showModelItemSelection(ModelTypeId modelTypeId, ModelVendorId modelVendorId, bool flag); // false代表为首次loadConfigFromJson,无需保存

    void initConfigModelPage();
    void initPrivateModelPage();
    void displayCloudModel();
    void displyPrivateModel();
    void switchLocalModel();
    void displyLocalModel();

    void addModelItem(ModelTypeId modelTypeId, ModelVendorId modelVendorId, bool flag);          // false代表为首次loadConfigFromJson,无需保存
    void configCurrentCapabilityModelKeys();
    void resetCurrentCapabilityModelKeys(Capability capability, const char *model);
    void switchSelectCloudModel(ModelTypeId modelTypeId, ModelVendorId modelVendorId);
    void resetSelectCloudModel(ModelTypeId modelTypeId, ModelVendorId modelVendorId);

    QJsonObject mapToJson(const QMap<ModelVendorId, S_Model>& map);  // 将QMap转换为JSON
    QMap<ModelVendorId, S_Model> jsonToMap(const QJsonObject& obj);  // 将JSON转换为QMap
    void saveConfigToJson();                                         // 保存数据到JSON文件
    void loadConfigFromJson();                                       // 从JSON文件中加载数据


private:
    class Private;
    shared_ptr<Private> d_ptr;
};

#endif // KAMULTIMODELSETTINGPAGE_H

